@@ -1,9 +1,9 @@
 /*@z08.c:Object Manifest:ReplaceWithSplit()@**********************************/
 /*                                                                           */
-/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.06)                       */
-/*  COPYRIGHT (C) 1994 Jeffrey H. Kingston                                   */
+/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.08)                       */
+/*  COPYRIGHT (C) 1991, 1996 Jeffrey H. Kingston                             */
 /*                                                                           */
-/*  Jeffrey H. Kingston (jeff@cs.su.oz.au)                                   */
+/*  Jeffrey H. Kingston (jeff@cs.usyd.edu.au)                                */
 /*  Basser Department of Computer Science                                    */
 /*  The University of Sydney 2006                                            */
 /*  AUSTRALIA                                                                */
@@ -34,19 +34,22 @@
 /*                                                                           */
 /*  static SetUnderline(x)                                                   */
 /*                                                                           */
-/*  Set underline() flags in object x as appropriate.                        */
+/*  Set underline() flags in object x to TRUE as appropriate.                */
 /*                                                                           */
 /*****************************************************************************/
 
 static void SetUnderline(OBJECT x)
 { OBJECT y, link;
+  debug2(DOM, D, "  Manifest underlining %s %s", Image(type(x)),EchoObject(x));
   if( type(x) == ACAT )
   { for( link = Down(x);  link != x;  link = NextDown(link) )
     { Child(y, link);
       SetUnderline(y);
     }
   }
-  else underline(x) = TRUE;
+  debug3(DOM, DDD, "  SetUnderline underline() := %s for %s %s",
+    bool(TRUE), Image(type(x)), EchoObject(x));
+  underline(x) = TRUE;
 } /* end SetUnderline */
 
 
@@ -60,19 +63,19 @@ static void SetUnderline(OBJECT x)
 /*****************************************************************************/
 
 #define ReplaceWithSplit(x, bthr, fthr)					\
-   if( bthr[ROW] || bthr[COL] || fthr[ROW] || fthr[COL] )		\
+   if( bthr[ROWM] || bthr[COLM] || fthr[ROWM] || fthr[COLM] )		\
 	x = insert_split(x, bthr, fthr)
 
 static OBJECT insert_split(OBJECT x, OBJECT bthr[2], OBJECT fthr[2])
 { OBJECT res, new_op;  int dim;
   debug1(DOM, DD, "ReplaceWithSplit(%s, -)", EchoObject(x));
   assert( type(x) != SPLIT, "ReplaceWithSplit: type(x) already SPLIT!" );
-  res = New(SPLIT);
+  New(res, SPLIT);
   FposCopy(fpos(res), fpos(x));
   ReplaceNode(res, x);
-  for( dim = COL;  dim <= ROW;  dim++ )
+  for( dim = COLM;  dim <= ROWM;  dim++ )
   { if( bthr[dim] || fthr[dim] )
-    { new_op = New(dim == COL ? COL_THR : ROW_THR);
+    { New(new_op, dim == COLM ? COL_THR : ROW_THR);
       thr_state(new_op) = NOTSIZED;
       fwd(new_op, 1-dim) = 0;	/* will hold max frame_size */
       back(new_op, 1-dim) = 0;	/* will hold max frame_origin */
@@ -148,7 +151,7 @@ OBJECT ReplaceWithTidy(OBJECT x, BOOLEAN one_word)
 	    { tmp = MakeWord(buff_typ, buff, &buff_pos);
 	      buff_len = 0;  buff_typ = WORD;
 	      if( res == nilobj )
-	      { res = New(ACAT);
+	      { New(res, ACAT);
 		FposCopy(fpos(res), fpos(x));
 	      }
 	      Link(res, tmp);  Link(res, y);
@@ -234,8 +237,19 @@ static OBJECT ManifestCat(OBJECT x, OBJECT env, STYLE *style, OBJECT bthr[2],
 OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
 { OBJECT bt[2], ft[2], y, link, gaplink, g, first_bt, last_ft, z;
   int par, perp;  unsigned res_inc;  BOOLEAN still_backing;
+  STYLE new_style;
     
-  par = type(x) == HCAT ? ROW : COL;
+  StyleCopy(new_style, *style);
+  if( type(x) == HCAT )
+  { par = ROWM;
+    adjust_cat(x) = hadjust(*style);
+    hadjust(new_style) = FALSE;
+  }
+  else
+  { par = COLM;
+    adjust_cat(x) = vadjust(*style);
+    vadjust(new_style) = FALSE;
+  }
   perp = 1 - par;
   link = Down(x);
   gaplink = NextDown(link);
@@ -245,14 +259,19 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
   /* set bt and ft threads for y */
   bt[perp] = bthr[perp];
   ft[perp] = fthr[perp];
-  first_bt = bt[par] = bthr[par] ? New(THREAD) : nilobj;
-  ft[par] = join(gap(g)) ? New(THREAD) : nilobj;
+  if( bthr[par] )  { New(first_bt, THREAD); }
+  else first_bt = nilobj;
+  bt[par] = first_bt;
+  /* *** first_bt = bt[par] = bthr[par] ? New(THREAD) : nilobj; *** */
+  if( join(gap(g)) )  { New(ft[par], THREAD); }
+  else ft[par] = nilobj;
+  /* *** ft[par] = join(gap(g)) ? New(THREAD) : nilobj;         *** */
   still_backing = first_bt != nilobj;
 
   /* manifest y and insinuate any cross-references */
-  y = Manifest(y, env, style, bt, ft, target, crs, ok, FALSE);
+  y = Manifest(y, env, &new_style, bt, ft, target, crs, ok, FALSE);
   if( type(x) == VCAT && ok && *crs != nilobj )
-  { debug1(DCR, D, "  insinuating %s", EchoObject(*crs));
+  { debug1(DCR, DD, "  insinuating %s", EchoObject(*crs));
     TransferLinks(Down(*crs), *crs, link);
     DisposeObject(*crs);
     *crs = nilobj;
@@ -266,7 +285,7 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
     assert( Down(g) != g, "Manifest/VCAT: GAP_OBJ has no child!" );
     Child(z, Down(g));
     debug1(DOM, DD, "manifesting gap, style = %s", EchoStyle(style));
-    z = Manifest(z, env, style, nbt, nft, &ntarget, crs, FALSE, FALSE);
+    z = Manifest(z, env, &new_style, nbt, nft, &ntarget, crs, FALSE, FALSE);
     debug1(DOM, DD, "replacing with tidy, style = %s", EchoStyle(style));
     z = ReplaceWithTidy(z, FALSE);
     debug1(DOM, DD, "calling GetGap, style = %s", EchoStyle(style));
@@ -284,14 +303,25 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
 
     /* set bt and ft threads for y */
     last_ft = ft[par];
+
+    /* ***
     bt[par] = ft[par] ? New(THREAD) : nilobj;
     ft[par] = g != nilobj ? join(gap(g)) ? New(THREAD) : nilobj
 			  : fthr[par]    ? New(THREAD) : nilobj;
+    *** */
+    if( ft[par] ) { New(bt[par], THREAD); }  else bt[par] = nilobj;
+    if( g != nilobj )
+    { if( join(gap(g)) )  { New(ft[par], THREAD); }  else ft[par] = nilobj;
+    }
+    else
+    {
+      if( fthr[par] )  { New(ft[par], THREAD); }  else ft[par] = nilobj;
+    }
 
     /* manifest y and insinuate any cross references */
-    y = Manifest(y, env, style, bt, ft, target, crs, ok, FALSE);
+    y = Manifest(y, env, &new_style, bt, ft, target, crs, ok, FALSE);
     if( type(x) == VCAT && ok && *crs != nilobj )
-    { debug1(DCR, D, "  insinuating %s", EchoObject(*crs));
+    { debug1(DCR, DD, "  insinuating %s", EchoObject(*crs));
       TransferLinks(Down(*crs), *crs, link);
       DisposeObject(*crs);
       *crs = nilobj;
@@ -373,7 +403,7 @@ OBJECT fthr[2], OBJECT *target,  OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
   /* make sure the right parameter is an ACAT */
   Child(y, LastDown(x));
   if( type(y) == YIELD )
-  { z = New(ACAT);
+  { New(z, ACAT);
     MoveLink(Up(y), z, PARENT);
     Link(x, z);
     y = z;
@@ -509,9 +539,14 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
     cross_type(y) = CROSS_PREC;
   else if( is_word(type(z)) && StringEqual(string(z), KW_FOLLOWING) )
     cross_type(y) = CROSS_FOLL;
+  else if( is_word(type(z)) && StringEqual(string(z), KW_FOLL_OR_PREC) )
+  {
+    assert(FALSE, "ManifestTg: CROSS_FOLL_OR_PREC not done yet!");
+    cross_type(y) = CROSS_FOLL_OR_PREC;
+  }
   else
-  { Error(8, 16, "%s or %s expected in left parameter of %s",
-      WARN, &fpos(z), KW_PRECEDING, KW_FOLLOWING, KW_TAGGED);
+  { Error(8, 16, "%s, %s or %s expected in left parameter of %s",
+      WARN, &fpos(z), KW_PRECEDING, KW_FOLLOWING, KW_FOLL_OR_PREC, KW_TAGGED);
     y = MakeWord(WORD, STR_EMPTY, &fpos(x));
     ReplaceNode(y, x);  DisposeObject(x);
     x = Manifest(y, env, style, bthr, fthr, target, crs, ok, FALSE);
@@ -539,7 +574,7 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
   DisposeObject(x);
   x = y;
   ReplaceWithSplit(x, bthr, fthr);
-  debug1(DCR, D, "  tagged manifesting %s", EchoObject(x));
+  debug1(DCR, DD, "  tagged manifesting %s", EchoObject(x));
   return x;
 } /* end ManifestTg */
 
@@ -559,7 +594,7 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
 
   sym = actual(x);
   StyleCopy(save_style(x), *style);
-  debug1(DOM, D,  "  [ manifesting closure %s", SymName(sym));
+  debug1(DOM, DD,  "  [ manifesting closure %s", SymName(sym));
 
   /* expand parameters where possible, and find if they are all free */
   symbol_free = TRUE;
@@ -635,10 +670,10 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
     if( y != nilobj && type(y) == CLOSURE )
     { debug0(DCR, DDD, "calling SetEnv from Manifest (a)");
       env = SetEnv(y, nilobj);
-      hold_env2 = New(ACAT);  Link(hold_env2, env);
+      New(hold_env2, ACAT);  Link(hold_env2, env);
     }
     else
-    { Error(8, 22, "symbol %s used outside %s", WARN, &fpos(x), SymName(sym),
+    { Error(8, 18, "symbol %s used outside %s", WARN, &fpos(x), SymName(sym),
 	SymName(enclosing(sym)));
       hold_env2 = nilobj;
     }
@@ -648,10 +683,13 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
   if( has_target(sym) && !need_expand )
   {
     /* convert symbols with targets to unsized galleys */
-    OBJECT hd = New(HEAD);
+    OBJECT hd;
+    New(hd, HEAD);
+    limiter(hd) = opt_components(hd) = opt_constraints(hd) = nilobj;
     FposCopy(fpos(hd), fpos(x));
+    gall_dir(hd) = horiz_galley(sym);
     actual(hd) = sym;
-    backward(hd) = TargetSymbol(x, &whereto(hd));
+    foll_or_prec(hd) = TargetSymbol(x, &whereto(hd));
     ready_galls(hd) = nilobj;
     must_expand(hd) = TRUE;
     sized(hd) = FALSE;
@@ -659,7 +697,7 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
     Link(hd, x);
     AttachEnv(env, x);
     x = hd;
-    threaded(x) = bthr[COL] != nilobj || fthr[COL] != nilobj;
+    threaded(x) = bthr[COLM] != nilobj || fthr[COLM] != nilobj;
     ReplaceWithSplit(x, bthr, fthr);
   }
   else if(
@@ -678,24 +716,21 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
     x = ClosureExpand(x, env, ok, crs, &res_env);
     *** */
     x = ClosureExpand(x, env, TRUE, crs, &res_env);
-    hold_env = New(ACAT);  Link(hold_env, res_env);
+    New(hold_env, ACAT);  Link(hold_env, res_env);
     debug1(DOM, DD, "recursive call; style: %s", EchoStyle(style));
     if( type(x) == FILTERED )
     { assert( type(sym) == RPAR, "ManifestCl/filtered: type(sym)!" );
       assert( filter(enclosing(sym)) != nilobj, "ManifestCl filter-encl!" );
-      command = New(CLOSURE);
+      New(command, CLOSURE);
       FposCopy(fpos(command), fpos(x));
       actual(command) = filter(enclosing(sym));
       FilterSetFileNames(x);
       command = Manifest(command,env,style,nbt,nft,&ntarget,crs,FALSE,FALSE);
       command = ReplaceWithTidy(command, TRUE);
       if( !is_word(type(command)) )
-	Error(8, 23, "filter parameter of %s symbol is not simple",
+	Error(8, 19, "filter parameter of %s symbol is not simple",
 	  FATAL, &fpos(command), SymName(enclosing(sym)));
       y = FilterExecute(x, string(command), res_env);
-      /* ***
-      y = FilterExecute(x, string(command), env);
-      *** */
       DisposeObject(command);
       ReplaceNode(y, x);
       DisposeObject(x);
@@ -708,12 +743,12 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
   {
     /* indefinite symbol, leave unexpanded */
     AttachEnv(env, x);
-    threaded(x) = bthr[COL] != nilobj || fthr[COL] != nilobj;
+    threaded(x) = bthr[COLM] != nilobj || fthr[COLM] != nilobj;
     debug0(DOM, DD,  "  closure; calling ReplaceWithSplit");
     ReplaceWithSplit(x, bthr, fthr);
   }
   if( hold_env2 != nilobj )  DisposeObject(hold_env2);
-  debug0(DOM, D,  "  ] returning from manifesting closure");
+  debug0(DOM, DD,  "  ] returning from manifesting closure");
   return x;
 } /* end ManifestCl */
 
@@ -735,10 +770,10 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
 /*  Manifest will construct threads and pass them up as children of bthr[]   */
 /*  and fthr[] whenever non-nilobj values of these variables are passed in:  */
 /*                                                                           */
-/*      bthr[COL]            protrudes upwards from x                        */
-/*      fthr[COL]            protrudes downwards from x                      */
-/*      bthr[ROW]            protrudes leftwards from x                      */
-/*      fthr[ROW]            protrudes rightwards from x                     */
+/*      bthr[COLM]           protrudes upwards from x                        */
+/*      fthr[COLM]           protrudes downwards from x                      */
+/*      bthr[ROWM]           protrudes leftwards from x                      */
+/*      fthr[ROWM]           protrudes rightwards from x                     */
 /*                                                                           */
 /*  If *target != nilobj, Manifest will expand indefinite closures leading   */
 /*  to the first @Galley lying within an object of type *target.             */
@@ -754,20 +789,29 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
 /*  @Wide and similar operators, and the gap following a concatenation       */
 /*  operator, are examples of such non-real objects.  The ok flag is true    */
 /*  when x is part of a real object.  This is needed because some things,    */
-/*  such as the insinuation of cross references and the breaking of          */
-/*  lines @Break ACAT objects, only apply to real objects.                   */
+/*  such as the insinuation of cross references, the breaking of             */
+/*  lines @Break ACAT objects, and conversion to small capitals, only apply  */
+/*  to real objects.                                                         */
 /*                                                                           */
 /*  If *crs != nilobj, it points to a list of indexes to cross-references    */
 /*  which are to be insinuated into the manifested form of x if x is real.   */
 /*                                                                           */
 /*  If need_expand is TRUE it forces closure x to expand.                    */
 /*                                                                           */
+/*  A postcondition of Manifest() is that the underline() flag is set to     */
+/*  either TRUE or FALSE in every WORD, every QWORD, and every child of      */
+/*  every ACAT, including the gaps.  This can be verified by checking that   */
+/*  the WORD and QWORD cases set underline() to FALSE, and the ACAT case     */
+/*  sets every child of the ACAT to FALSE.  To see that the correct subset   */
+/*  of these flags gets changed to TRUE, consult SetUnderline().  The        */
+/*  underline() flag is undefined otherwise.                                 */
+/*                                                                           */
 /*****************************************************************************/
 #define MAX_DEPTH 500
 
 OBJECT Manifest(OBJECT x, OBJECT env, STYLE *style, OBJECT bthr[2],
 OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
-{ OBJECT bt[2], ft[2], y, link, gaplink, g;
+{ OBJECT bt[2], ft[2], y, link, gaplink, g;  register FULL_CHAR *p;
   OBJECT res, res_env, res_env2, hold_env, hold_env2, z, prev;
   OBJECT link1, link2, x1, x2, y1, y2;
   int par;  GAP res_gap;  unsigned res_inc;  STYLE new_style;
@@ -776,19 +820,14 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
   static OBJECT mo[MAX_DEPTH];  static int depth = 0;
   mo[depth++] = x;
   if( depth == MAX_DEPTH )
-  { Error(8, 18, "maximum depth of symbol expansion (%d) reached",
+  { Error(8, 20, "maximum depth of symbol expansion (%d) reached",
       WARN, &fpos(x), MAX_DEPTH);
-    Error(8, 19, "the symbols currently being expanded are:", WARN, &fpos(x));
+    Error(8, 21, "the symbols currently being expanded are:", WARN, &fpos(x));
     while( --depth >= 0 )
     {
-      Error(8, 20, "at %d: %s", WARN, &fpos(x), depth, EchoObject(mo[depth]));
-      /* ***
-      Error(8, 20, "at %d: %s", WARN, &fpos(x), depth,
-        type(mo[depth]) == CLOSURE ? SymName(actual(mo[depth])) :
-        is_word(type(mo[depth])) ? string(mo[depth]) : Image(type(mo[depth])));
-      *** */
+      Error(8, 22, "at %d: %s", WARN, &fpos(x), depth, EchoObject(mo[depth]));
     }
-    Error(8, 21, "exiting now", FATAL, &fpos(x));
+    Error(8, 23, "exiting now", FATAL, &fpos(x));
   }
 #endif
 
@@ -796,8 +835,8 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
   debug1(DOM, DD,  "  environment: %s", EchoObject(env));
   debug6(DOM, DD,  "  style: %s;  target: %s;  threads: %s%s%s%s",
 	EchoStyle(style), SymName(*target),
-	bthr[COL] ? " up"    : "",  fthr[COL] ? " down"  : "",
-	bthr[ROW] ? " left"  : "",  fthr[ROW] ? " right" : "");
+	bthr[COLM] ? " up"    : "",  fthr[COLM] ? " down"  : "",
+	bthr[ROWM] ? " left"  : "",  fthr[ROWM] ? " right" : "");
 
   switch( type(x) )
   {
@@ -852,11 +891,11 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
 	*** */
 	x = CrossExpand(x, env, style, crs, &res_env);
         assert( type(x) == CLOSURE, "Manifest/CROSS: type(x)!" );
-        hold_env = New(ACAT);  Link(hold_env, res_env);
+        New(hold_env, ACAT);  Link(hold_env, res_env);
         /* expand here (calling Manifest immediately makes unwanted cr) */
         debug0(DCE, DD, "  calling ClosureExpand from Manifest/CROSS");
         x = ClosureExpand(x, res_env, FALSE, crs, &res_env2);
-        hold_env2 = New(ACAT);  Link(hold_env2, res_env2);
+        New(hold_env2, ACAT);  Link(hold_env2, res_env2);
         x = Manifest(x, res_env2, style, bthr, fthr, target, crs, ok, TRUE);
         DisposeObject(hold_env);
         DisposeObject(hold_env2);
@@ -878,12 +917,14 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
 	word_colour(x) = colour(*style);
 	word_language(x) = language(*style);
 	word_hyph(x) = hyph_style(*style) == HYPH_ON;
+	debug3(DOM, DDD, "  manfifest/WORD underline() := %s for %s %s",
+	  bool(FALSE), Image(type(x)), EchoObject(x));
 	underline(x) = FALSE;
-	if( small_caps(*style) )  x = SmallCaps(x, style);
+	if( small_caps(*style) && ok )  x = MapSmallCaps(x, style);
 	ReplaceWithSplit(x, bthr, fthr);
 	break;
       }
-      y = New(ACAT);
+      New(y, ACAT);
       FposCopy(fpos(y), fpos(x));
       ReplaceNode(y, x);
       Link(y, x);  x = y;
@@ -893,6 +934,9 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
     case ACAT:
     
       StyleCopy(save_style(x), *style);
+      adjust_cat(x) = padjust(*style);
+      StyleCopy(new_style, *style);
+      padjust(new_style) = FALSE;
       assert(Down(x) != x, "Manifest: ACAT!" );
       link = Down(x);  Child(y, link);
       assert( type(y) != GAP_OBJ, "Manifest ACAT: GAP_OBJ is first!" );
@@ -904,24 +948,29 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
 	word_colour(y) = colour(*style);
 	word_language(y) = language(*style);
 	word_hyph(y) = hyph_style(*style) == HYPH_ON;
-	if( small_caps(*style) )  y = SmallCaps(y, style);
+	if( small_caps(*style) && ok )  y = MapSmallCaps(y, style);
       }
-      else y = Manifest(y, env, style, nbt, nft, target, crs, ok, FALSE);
+      else y = Manifest(y, env, &new_style, nbt, nft, target, crs, ok, FALSE);
+      debug3(DOM, DDD, "  manfifest/ACAT1 underline() := %s for %s %s",
+	bool(FALSE), Image(type(y)), EchoObject(y));
       underline(y) = FALSE;
       /* ??? if( is_word(type(y)) ) */
       if( ok && *crs != nilobj )
       {	
-	debug1(DCR, D, "  insinuating %s", EchoObject(*crs));
+	debug1(DCR, DD, "  insinuating %s", EchoObject(*crs));
 	TransferLinks(Down(*crs), *crs, link);
 	DisposeObject(*crs);
 	*crs = nilobj;
       }
       prev = y;
 
+      /* manifest subsequent gaps and children */
       for( gaplink = Down(link);  gaplink != x;  gaplink = NextDown(link) )
       {
 	Child(g, gaplink);
 	assert( type(g) == GAP_OBJ, "Manifest ACAT: no GAP_OBJ!" );
+        debug3(DOM, DDD, "  manfifest/ACAT2 underline() := %s for %s %s",
+	  bool(FALSE), Image(type(g)), EchoObject(g));
         underline(g) = FALSE;
 	link = NextDown(gaplink);
 	assert( link != x, "Manifest ACAT: GAP_OBJ is last!" );
@@ -935,9 +984,11 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
 	  word_colour(y) = colour(*style);
 	  word_language(y) = language(*style);
 	  word_hyph(y) = hyph_style(*style) == HYPH_ON;
-	  if( small_caps(*style) )  y = SmallCaps(y, style);
+	  if( small_caps(*style) && ok )  y = MapSmallCaps(y, style);
 	}
-	else y = Manifest(y, env, style, nbt, nft, target, crs, ok, FALSE);
+	else y = Manifest(y, env, &new_style, nbt, nft, target, crs, ok, FALSE);
+        debug3(DOM, DDD, "  manfifest/ACAT3 underline() := %s for %s %s",
+	  bool(FALSE), Image(type(y)), EchoObject(y));
         underline(y) = FALSE;
 
 	/* manifest the gap object */
@@ -945,7 +996,7 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
 	{
 	  /* explicit & operator whose value is the child of g */
 	  Child(z, Down(g));
-	  z = Manifest(z, env, style, nbt, nft, &ntarget, crs, FALSE, FALSE);
+	  z = Manifest(z, env, &new_style, nbt, nft, &ntarget, crs, FALSE, FALSE);
 	  z = ReplaceWithTidy(z, FALSE);
 	  GetGap(z, style, &gap(g), &res_inc);
 	  vspace(g) = hspace(g) = 0;
@@ -954,7 +1005,94 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
 	{
 	  /* implicit & operator */
 	  GapCopy(gap(g), space_gap(*style));
-	  width(gap(g)) = width(gap(g)) * (vspace(g) + hspace(g));
+	  switch( space_style(*style) )
+	  {
+	    case SPACE_LOUT:
+
+	      /* usual Lout spacing, the number of white space characters */
+	      width(gap(g)) = width(gap(g)) * (vspace(g) + hspace(g));
+	      break;
+
+
+	    case SPACE_COMPRESS:
+
+	      /* either zero or one space */
+	      if( vspace(g) + hspace(g) == 0 )
+		width(gap(g)) = 0;
+	      /* else width is like one space, so OK as is */
+	      break;
+
+
+	    case SPACE_SEPARATE:
+
+	      /* exactly one space always, so do nothing further */
+	      break;
+
+
+	    case SPACE_TROFF:
+
+	      /* Lout spacing plus one extra space for sentence end at eoln */
+	      width(gap(g)) = width(gap(g)) * (vspace(g) + hspace(g));
+	      debugcond2(DLS, D, vspace(g) > 0, "  prev = %s %s",
+		Image(type(prev)), EchoObject(prev));
+	      if( vspace(g) > 0 )
+	      { 
+		/* set z to the preceding object; may need to search ACATs! */
+		z = prev;
+		while( type(z) == ACAT )
+		{ Child(z, LastDown(z));
+		}
+
+		/* if preceding object is a word, check for end sentence */
+		if( is_word(type(z)) )
+		{
+		  for( p = string(z);  *p != '\0';  p++ );
+		  debug4(DLS, D, "  prev = %s, last = %c, LSE = %s, LWES = %s",
+		    EchoObject(z), *(p-1), bool(LanguageSentenceEnds[*(p-1)]),
+		    bool(LanguageWordEndsSentence(z, FALSE)));
+		  if( p != string(z) && LanguageSentenceEnds[*(p-1)]
+		      && LanguageWordEndsSentence(z, FALSE) )
+		    width(gap(g)) += width(space_gap(*style));
+		}
+	      }
+	      break;
+
+
+	    case SPACE_TEX:
+
+	      if( vspace(g) + hspace(g) == 0 )
+	      {
+		/* zero spaces gives zero result, as for compress above */
+		width(gap(g)) = 0;
+	      }
+	      else
+	      {
+	        /* set z to the preceding object; may need to search ACATs! */
+	        z = prev;
+	        while( type(z) == ACAT )
+	        { Child(z, LastDown(z));
+	        }
+
+	        /* one extra space if preceding is word ending sentence */
+	        if( is_word(type(z)) )
+	        {
+		  for( p = string(z);  *p != '\0';  p++ );
+		  debug4(DLS, D, "  prev = %s, last = %c, LSE = %s, LWES = %s",
+		      EchoObject(z), *(p-1), bool(LanguageSentenceEnds[*(p-1)]),
+		      bool(LanguageWordEndsSentence(z, TRUE)));
+		  if( p != string(z) && LanguageSentenceEnds[*(p-1)]
+		      && LanguageWordEndsSentence(z, TRUE) )
+		    width(gap(g)) += width(space_gap(*style));
+	        }
+	      }
+	      break;
+
+
+	    default:
+
+	      assert(FALSE, "Manifest: unexpected space_style!");
+	      break;
+	  }
 	  if( vspace(g) > 0 && is_definite(type(y)) )  multiline = TRUE;
 	}
         debug1(DOM, DD, "  in ACAT, gap = %s", EchoLength(width(gap(g))));
@@ -966,6 +1104,7 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
 	    word_font(prev) == word_font(y) &&
 	    word_colour(prev) == word_colour(y) &&
 	    word_language(prev) == word_language(y) )
+	    /* no need to compare underline() since both are false */
 	{ unsigned typ;
 	  if( StringLength(string(prev))+StringLength(string(y)) >= MAX_BUFF )
 	    Error(8, 24, "word %s%s is too long",
@@ -977,7 +1116,11 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
 	  word_colour(y) = word_colour(prev);
 	  word_language(y) = word_language(prev);
 	  word_hyph(y) = word_hyph(prev);
+	  /* ***
 	  underline(y) = underline(prev);
+          debug3(DOM, DDD, "  manifest/ACAT4 underline() := %s for %s %s",
+	    bool(underline(y)), Image(type(y)), EchoObject(y));
+	  *** */
 	  MoveLink(link, y, CHILD);
 	  DisposeObject(z);
 	  DisposeChild(Up(prev));
@@ -988,46 +1131,64 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
 	/* insinuate any cross-references */
 	if( ok && *crs != nilobj )
 	{
-	  debug1(DCR, D, "  insinuating %s", EchoObject(*crs));
+	  debug1(DCR, DD, "  insinuating %s", EchoObject(*crs));
 	  TransferLinks(Down(*crs), *crs, link);
 	  DisposeObject(*crs);
 	  *crs = nilobj;
 	}
-
       }
 
       /* implement FILL_OFF break option if required */
       if( ok && multiline && fill_style(*style) == FILL_UNDEF )
 	Error(8, 25, "missing %s symbol or option", FATAL, &fpos(x), KW_BREAK);
       if( ok && multiline && fill_style(*style) == FILL_OFF )
-      {	OBJECT last_acat = x, new_acat;
-	x = New(VCAT);
-	ReplaceNode(x, last_acat);
-	Link(x, last_acat);
-	for( link = Down(last_acat); link != last_acat; link = NextDown(link) )
+      {	OBJECT prev_acat = x, new_acat;
+	New(x, VCAT);
+	adjust_cat(x) = FALSE;
+	ReplaceNode(x, prev_acat);
+	Link(x, prev_acat);
+	for( link = Down(prev_acat); link != prev_acat; link = NextDown(link) )
 	{ Child(g, link);
 	  if( type(g) == GAP_OBJ && mode(gap(g)) != NO_MODE && vspace(g) > 0 )
 	  { link = PrevDown(link);
 	    MoveLink(NextDown(link), x, PARENT);
 	    GapCopy(gap(g), line_gap(*style));
 	    width(gap(g)) *= vspace(g);
-	    new_acat = New(ACAT);
+	    New(new_acat, ACAT);
+	    adjust_cat(new_acat) = padjust(*style);
 	    FposCopy(fpos(new_acat), fpos(g));
 	    if( hspace(g) > 0 )
 	    { z = MakeWord(WORD, STR_EMPTY, &fpos(g));
+	      word_font(z) = font(*style);
+	      word_colour(z) = colour(*style);
+	      word_language(z) = language(*style);
+	      word_hyph(z) = hyph_style(*style) == HYPH_ON;
 	      Link(new_acat, z);
-	      z = New(GAP_OBJ);
+	      New(z, GAP_OBJ);
 	      hspace(z) = hspace(g);
 	      vspace(z) = 0;
+	      underline(z) = FALSE;
 	      GapCopy(gap(z), space_gap(*style));
 	      width(gap(z)) *= hspace(z);
 	      Link(new_acat, z);
 	    }
-	    TransferLinks(NextDown(link), last_acat, new_acat);
+	    TransferLinks(NextDown(link), prev_acat, new_acat);
 	    StyleCopy(save_style(new_acat), *style);
 	    Link(x, new_acat);
-	    last_acat = new_acat;
-	    link = last_acat;
+	    prev_acat = new_acat;
+	    link = prev_acat;
+	  }
+	}
+
+	/* remove any singleton ACAT objects under x, if they are VCATs */
+	for( link = Down(x);  link != x;  link = NextDown(link) )
+	{ Child(y, link);
+	  if( type(y) == ACAT && Down(y) == LastDown(y) )
+	  { Child(z, Down(y));
+	    if( type(z) == VCAT )
+	    { MoveLink(link, z, CHILD);
+	      DisposeObject(y);
+	    }
 	  }
 	}
       }
@@ -1087,16 +1248,12 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
     case VCONTRACT:
     case HEXPAND:
     case VEXPAND:
-    case PADJUST:
-    case HADJUST:
-    case VADJUST:
     case ONE_COL:
     case ONE_ROW:
     
       ETC:
       par = (type(x)==ONE_COL || type(x)==HEXPAND || type(x) == HCONTRACT ||
-	     type(x)==PADJUST || type(x)==HADJUST || type(x)==WIDE ||
-	     type(x)==HSHIFT) ? COL : ROW;
+	     type(x)==WIDE || type(x)==HSHIFT) ? COLM : ROWM;
       Child(y, Down(x));
 
       /* manifest the child, propagating perp threads and suppressing pars */
@@ -1134,6 +1291,8 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
 
     case HSCALE:
     case VSCALE:
+    case HCOVER:
+    case VCOVER:
 
       Child(y, Down(x));
       y = Manifest(y, env, style, nbt, nft, target, crs, ok, FALSE);
@@ -1224,16 +1383,19 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
 	  WARN, &fpos(y), KW_XCHAR);
 	res = MakeWord(WORD, STR_EMPTY, &fpos(x));
       }
-      else if( (ch=EvRetrieve(string(y), FontEncoding(word_font(y)))) == '\0' )
-      {	type(y) = QWORD;
-	Error(8, 32, "%s dropped (character %s unknown in font %s)",
-	  WARN, &fpos(y), KW_XCHAR, StringQuotedWord(y),
-	  FontFamilyAndFace(word_font(y)));
-	res = MakeWord(WORD, STR_EMPTY, &fpos(x));
-      }
       else
-      {	res = MakeWord(QWORD, STR_SPACE, &fpos(x));
-	string(res)[0] = ch;
+      { ch = MapCharEncoding(string(y), FontMapping(word_font(y), &fpos(y)));
+        if( ch == '\0' )
+        { type(y) = QWORD;
+	  Error(8, 32, "%s dropped (character %s unknown in font %s)",
+	    WARN, &fpos(y), KW_XCHAR, StringQuotedWord(y),
+	    FontFamilyAndFace(word_font(y)));
+	  res = MakeWord(WORD, STR_EMPTY, &fpos(x));
+        }
+        else
+        { res = MakeWord(QWORD, STR_SPACE, &fpos(x));
+	  string(res)[0] = ch;
+        }
       }
       ReplaceNode(res, x);
       DisposeObject(x);
@@ -1257,6 +1419,8 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
 
     case FONT:
     case SPACE:
+    case YUNIT:
+    case ZUNIT:
     case BREAK:
     case COLOUR:
     case LANGUAGE:
@@ -1265,12 +1429,31 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
       StyleCopy(new_style, *style);
       Child(y, Down(x));
       y = Manifest(y, env, style, nbt, nft, &ntarget, crs, FALSE, FALSE);
-      y = ReplaceWithTidy(y, (BOOLEAN) (type(x) == COLOUR) );
-      if(      type(x) == FONT   ) FontChange(&new_style, y);
-      else if( type(x) == SPACE  ) SpaceChange(&new_style, y);
-      else if( type(x) == BREAK  ) BreakChange(&new_style, y);
-      else if( type(x) == COLOUR ) ColourChange(&new_style, y);
-      else                         LanguageChange(&new_style, y);
+      y = ReplaceWithTidy(y, type(x) == COLOUR);
+      switch( type(x) )
+      {
+	case FONT:	FontChange(&new_style, y);
+			break;
+
+	case SPACE:	SpaceChange(&new_style, y);
+			break;
+
+	case YUNIT:	YUnitChange(&new_style, y);
+			break;
+
+	case ZUNIT:	ZUnitChange(&new_style, y);
+			break;
+
+	case BREAK:	BreakChange(&new_style, y);
+			break;
+	
+	case COLOUR:	ColourChange(&new_style, y);
+			break;
+
+	case LANGUAGE:	LanguageChange(&new_style, y);
+			break;
+
+      }
       DisposeChild(Down(x));
       Child(y, Down(x));
       y = Manifest(y, env, &new_style, bthr, fthr, target, crs, ok, FALSE);
@@ -1279,13 +1462,31 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
       break;
 
 
+    case PADJUST:
+    case HADJUST:
+    case VADJUST:
+
+      StyleCopy(new_style, *style);
+      if(      type(x) == VADJUST )  vadjust(new_style) = TRUE;
+      else if( type(x) == HADJUST )  hadjust(new_style) = TRUE;
+      else                           padjust(new_style) = TRUE;
+      Child(y, Down(x));
+      y = Manifest(y, env, &new_style, bthr, fthr, target, crs, ok, FALSE);
+      DeleteLink(Down(x));
+      MergeNode(y, x);  x = y;
+      break;
+
+
     case UNDERLINE:
 
+      /* change x to an ACAT and set the underline flags in its child */
       assert( Down(x) != x && NextDown(Down(x)) == x, "Manifest: UNDERLINE!" );
+      type(x) = ACAT;
+      adjust_cat(x) = padjust(*style);
+      padjust(*style) = FALSE;
+      StyleCopy(save_style(x), *style);
       Child(y, Down(x));
       y = Manifest(y, env, style, bthr, fthr, target, crs, ok, FALSE);
-      DeleteLink(Down(x));
-      MergeNode(y, x);  x = y;
       SetUnderline(x);
       break;
 
@@ -1299,7 +1500,10 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
       /* find the first child of x, make sure it is an ACAT, and manifest */
       Child(x1, Down(x));
       if( type(x1) != ACAT )
-      { OBJECT newx1 = New(ACAT);
+      { OBJECT newx1;
+	New(newx1, ACAT);
+        adjust_cat(newx1) = padjust(*style);
+	padjust(*style) = FALSE;
         MoveLink(Down(x), newx1, CHILD);
         Link(newx1, x1);
         x1 = newx1;
@@ -1319,7 +1523,10 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
       /* find the second child of x, make sure it is an ACAT, and manifest */
       Child(x2, NextDown(Down(x)));
       if( type(x2) != ACAT )
-      { OBJECT newx2 = New(ACAT);
+      { OBJECT newx2;
+	New(newx2, ACAT);
+        adjust_cat(newx2) = padjust(*style);
+	padjust(*style) = FALSE;
         MoveLink(Down(x), newx2, CHILD);
         Link(newx2, x2);
         x2 = newx2;
@@ -1398,6 +1605,29 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
       break;
 
 
+    case INSERT:
+
+      /* manifest and detach the left parameter, call it z */
+      Child(z, Down(x));
+      z = Manifest(z, env, style, nbt, nft, &ntarget, crs, FALSE, FALSE);
+      DeleteLink(Down(x));
+
+      /* manifest the right parameter and make it the result */
+      Child(y, LastDown(x));
+      y = Manifest(y, env, style, bthr, fthr, target, crs, ok, FALSE);
+      DeleteLink(LastDown(x));
+      MergeNode(y, x);  x = y;
+
+      /* now find the reattachment point for z down inside the result, x */
+      InsertObject(x, &z);
+      if( z != nilobj )
+      { Error(8, 34, "object dropped by %s: no suitable insert point", WARN,
+	  &fpos(x), KW_INSERT);
+	DisposeObject(z);
+      }
+      break;
+
+
     case NEXT:
 
       assert( Down(x) != x, "Manifest/NEXT: Down(x) == x!" );
@@ -1416,7 +1646,7 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
 
     case OPEN:
 
-      debug0(DCR, D, "  [ Manifest/OPEN begins:");
+      debug0(DCR, DD, "  [ Manifest/OPEN begins:");
       Child(y, Down(x));
       Child(res, LastDown(x));
       hold_env = nilobj;
@@ -1425,7 +1655,7 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
 	StyleCopy(save_style(y), *style);
 	debug0(DCR, DDD, "calling SetEnv from Manifest (b)");
 	res_env = SetEnv(y, nilobj);
-	hold_env = New(ACAT);  Link(hold_env, res_env);
+	New(hold_env, ACAT);  Link(hold_env, res_env);
 	res = Manifest(res, res_env, style, bthr, fthr, target, crs, ok, FALSE);
       }
       else if( type(y) == CROSS )
@@ -1439,29 +1669,30 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
 	  AttachEnv(res_env, y);
 	  debug0(DCR, DDD, "calling SetEnv from Manifest (c)");
 	  res_env = SetEnv(y, env);
-	  hold_env = New(ACAT);  Link(hold_env, res_env);
+	  New(hold_env, ACAT);  Link(hold_env, res_env);
 	  res = Manifest(res, res_env, style, bthr, fthr, target, crs, ok, FALSE);
 	}
 	else
-	{ Error(8, 34, "invalid left parameter of %s", WARN, &fpos(y), KW_OPEN);
+	{ Error(8, 35, "invalid left parameter of %s", WARN, &fpos(y), KW_OPEN);
 	  res = Manifest(res, env, style, bthr, fthr, target, crs, ok, FALSE);
 	}
       }
       else
-      {	Error(8, 34, "invalid left parameter of %s", WARN, &fpos(y), KW_OPEN);
+      {	Error(8, 36, "invalid left parameter of %s", WARN, &fpos(y), KW_OPEN);
 	res = Manifest(res, env, style, bthr, fthr, target, crs, ok, FALSE);
       }
       ReplaceNode(res, x);
       DisposeObject(x);
       if( hold_env != nilobj )  DisposeObject(hold_env);
       x = res;
-      debug0(DCR, D, "  ] Manifest/OPEN ends");
+      debug0(DCR, DD, "  ] Manifest/OPEN ends");
       break;
 
 
     case TAGGED:
 
       x = ManifestTg(x, env, style, bthr, fthr, target, crs, ok, need_expand);
+      debug2(DCR, DD, "Manifest returning %ld %s", (long) x, EchoObject(x));
       break;
 
 
@@ -1485,7 +1716,7 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
       y = Manifest(y, env, style, nbt, nft, &ntarget, crs, FALSE, FALSE);
       y = ReplaceWithTidy(y, FALSE);
       if( !is_word(type(y)) )
-      { Error(8, 35, "%s deleted (invalid right parameter)", WARN, &fpos(y),
+      { Error(8, 37, "%s deleted (invalid right parameter)", WARN, &fpos(y),
 	  type(x) == INCGRAPHIC ? KW_INCGRAPHIC : KW_SINCGRAPHIC);
 	y = MakeWord(WORD, STR_EMPTY, &fpos(x));
 	ReplaceNode(y, x);  DisposeObject(x);
@@ -1498,17 +1729,17 @@ OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand)
 
     default:
 
-      Error(8, 36, "Manifest: %s", INTERN, &fpos(x), Image(type(x)));
+      assert1(FALSE, "Manifest:", Image(type(x)));
       break;
 
   } /* end switch */
 
   debug2(DOM, DD, "]Manifest returning %s %s", Image(type(x)), EchoObject(x));
   debug1(DOM, DD, "  at exit, style = %s", EchoStyle(style));
-  debug1(DOM, DDD, "up:    ", EchoObject(bthr[COL]));
-  debug1(DOM, DDD, "down:  ", EchoObject(fthr[COL]));
-  debug1(DOM, DDD, "left:  ", EchoObject(bthr[ROW]));
-  debug1(DOM, DDD, "right: ", EchoObject(fthr[ROW]));
+  debug1(DOM, DDD, "up:    ", EchoObject(bthr[COLM]));
+  debug1(DOM, DDD, "down:  ", EchoObject(fthr[COLM]));
+  debug1(DOM, DDD, "left:  ", EchoObject(bthr[ROWM]));
+  debug1(DOM, DDD, "right: ", EchoObject(fthr[ROWM]));
 #if DEBUG_ON
   depth--;
 #endif
