@@ -1,9 +1,9 @@
 /*@z19.c:Galley Attaching:DetachGalley()@*************************************/
 /*                                                                           */
-/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.06)                       */
-/*  COPYRIGHT (C) 1994 Jeffrey H. Kingston                                   */
+/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.08)                       */
+/*  COPYRIGHT (C) 1991, 1996 Jeffrey H. Kingston                             */
 /*                                                                           */
-/*  Jeffrey H. Kingston (jeff@cs.su.oz.au)                                   */
+/*  Jeffrey H. Kingston (jeff@cs.usyd.edu.au)                                */
 /*  Basser Department of Computer Science                                    */
 /*  The University of Sydney 2006                                            */
 /*  AUSTRALIA                                                                */
@@ -32,48 +32,56 @@
 
 /*****************************************************************************/
 /*                                                                           */
-/*  OBJECT InterposeVScale(y, scale_factor)                                  */
+/*  OBJECT InterposeScale(y, scale_factor, dim)                              */
 /*                                                                           */
-/*  Interpose a @VScale symbol above y with the given scale factor.          */
+/*  Interpose a @Scale symbol above y with the given scale factor.           */
 /*                                                                           */
 /*****************************************************************************/
 
-static OBJECT InterposeVScale(OBJECT y, int scale_factor)
+static OBJECT InterposeScale(OBJECT y, int scale_factor, int dim)
 { OBJECT res;
-  res = New(VSCALE);
+  New(res, SCALE);
   FposCopy(fpos(res), fpos(y));
-  back(res, COL) = back(y, COL);
-  fwd(res, COL)  = fwd(y, COL);
-  back(res, ROW) = (back(y, ROW) * scale_factor) / SF;
-  fwd(res, ROW)  = (fwd(y, ROW) * scale_factor) / SF;
+  if( dim == COLM )
+  { bc(constraint(res)) = scale_factor;
+    fc(constraint(res)) = 1 * SF;
+  }
+  else
+  { bc(constraint(res)) = 1 * SF;
+    fc(constraint(res)) = scale_factor;
+  }
+  back(res, dim) = (back(y, dim) * scale_factor) / SF;
+  fwd(res, dim)  = (fwd(y, dim) * scale_factor) / SF;
+  back(res, 1-dim) = back(y, 1-dim);
+  fwd(res, 1-dim)  = fwd(y, 1-dim);
   ReplaceNode(res, y);
   Link(res, y);
   return res;
-} /* end InterposeVScale */
+} /* end InterposeScale */
 
 
 /*****************************************************************************/
 /*                                                                           */
-/*  OBJECT InterposeHigh(y)                                                  */
+/*  OBJECT InterposeWideOrHigh(y, dim)                                       */
 /*                                                                           */
-/*  Interpose a @High symbol above y with the same size as y, with a value   */
-/*  which prevents any further increase in the size of y.                    */
+/*  Interpose a @Wide or @High symbol above y with the same size as y, with  */
+/*  a value which prevents any further increase in the size of y.            */
 /*                                                                           */
 /*****************************************************************************/
 
-static OBJECT InterposeHigh(OBJECT y)
+static OBJECT InterposeWideOrHigh(OBJECT y, int dim)
 { OBJECT res;
-  res = New(HIGH);
+  New(res, dim == COLM ? WIDE : HIGH);
   FposCopy(fpos(res), fpos(y));
-  back(res, COL) = back(y, COL);
-  fwd(res, COL)  = fwd(y, COL);
-  back(res, ROW) = back(y, ROW);
-  fwd(res, ROW)  = fwd(y, ROW);
-  SetConstraint(constraint(res), MAX_LEN, size(res, ROW), MAX_LEN);
+  back(res, dim) = back(y, dim);
+  fwd(res, dim)  = fwd(y, dim);
+  back(res, 1-dim) = back(y, 1-dim);
+  fwd(res, 1-dim)  = fwd(y, 1-dim);
+  SetConstraint(constraint(res), MAX_LEN, size(res, dim), MAX_LEN);
   ReplaceNode(res, y);
   Link(res, y);
   return res;
-} /* end InterposeHigh */
+} /* end InterposeWideOrHigh */
 
 
 /*****************************************************************************/
@@ -90,7 +98,7 @@ void DetachGalley(OBJECT hd)
   debug1(DGA, D, "DetachGalley( %s )", SymName(actual(hd)));
   Parent(prnt, Up(hd));
   assert( Up(prnt) != prnt, "DetachGalley: parent!" );
-  index = New(UNATTACHED);
+  New(index, UNATTACHED);
   pinpoint(index) = nilobj;
   MoveLink(Up(hd), index, PARENT);
   Link(NextDown(Up(prnt)), index);
@@ -223,6 +231,9 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
   CONSTRAINT c;			/* temporary variable holding a constraint   */
   OBJECT env, n1, tmp, zlink, z, sym;	/* placeholders and temporaries	     */
   BOOLEAN was_sized;		/* true if sized(hd) initially               */
+  int dim;			/* the galley direction                      */
+  LENGTH perp_back, perp_fwd;
+  OBJECT why, junk;
 
   debug2(DGA, D, "[ AttachGalley(Galley %s into %s)",
 	SymName(actual(hd)), SymName(whereto(hd)));
@@ -232,6 +243,7 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
   assert( type(hd_index) == UNATTACHED, "AttachGalley: not UNATTACHED!" );
   hd_inners = tg_inners = nilobj;
   was_sized = sized(hd);
+  dim = gall_dir(hd);
 
   for(;;)
   {
@@ -255,7 +267,7 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
 	/* search failed to find any new target, so kill the galley */
 	for( link = Down(hd); link != hd; link = NextDown(link) )
 	{ Child(y, link);
-	  if( type(y) == SPLIT )  Child(y, DownDim(y, ROW));
+	  if( type(y) == SPLIT )  Child(y, DownDim(y, dim));
 	  if( is_definite(type(y)) )  break;
 	}
 	if( link != hd )
@@ -263,7 +275,7 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
 	    WARN, &fpos(y), SymName(actual(hd)));
 	if( hd_inners != nilobj )  DisposeObject(hd_inners), hd_inners=nilobj;
 	if( tg_inners != nilobj )  DisposeObject(tg_inners), tg_inners=nilobj;
-	KillGalley(hd);
+	KillGalley(hd, FALSE);
 	*inners = nilobj;
 	debug0(DGA, D, "] AttachGalley returning ATTACH_KILLED");
 	return ATTACH_KILLED;
@@ -281,7 +293,7 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
     }
     else /* unsized galley, either backwards or normal */
     {
-      if( backward(hd) )
+      if( foll_or_prec(hd) == GALL_PREC )
       {	target_index= SearchGalley(Up(hd_index), sym, FALSE, TRUE,TRUE,FALSE);
 	need_precedes = FALSE;
       }
@@ -307,27 +319,43 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
     debug1(DYY, D, "[ EnterErrorBlock(FALSE) (expanding target %s)",
       SymName(actual(target)));
     EnterErrorBlock(FALSE);
-    target_galley = New(HEAD);
+    New(target_galley, HEAD);
+    limiter(target_galley) = nilobj;
+    opt_components(target_galley) = opt_constraints(target_galley) = nilobj;
+    gall_dir(target_galley) = external_hor(target) ? COLM : ROWM;
     FposCopy(fpos(target_galley), fpos(target));
     actual(target_galley) = actual(target);
     whereto(target_galley) = ready_galls(target_galley) = nilobj;
-    backward(target_galley) = must_expand(target_galley) = FALSE;
+    foll_or_prec(target_galley) = GALL_FOLL;
+    must_expand(target_galley) = FALSE;
     sized(target_galley) = FALSE;
-    Constrained(target, &c, COL);
-    if( !constrained(c) )
-      Error(19, 2, "receptive symbol %s has unconstrained width",
-	FATAL, &fpos(target), SymName(actual(target)));
-    debug2(DSC, DD, "Constrained( %s, COL ) = %s",
+
+    /* get perpendicular constraint (none if horizontal galley) */
+    if( dim == ROWM )
+    {
+      Constrained(target, &c, 1-dim, &junk);
+      if( !constrained(c) )
+        Error(19, 2, "receptive symbol %s has unconstrained width",
+	  FATAL, &fpos(target), SymName(actual(target)));
+      debug2(DSC, DD, "Constrained( %s, 1-dim ) = %s",
 	EchoObject(target), EchoConstraint(&c));
+      if( !FitsConstraint(0, 0, c) )
+      { debug0(DGA, D, "  reject: target_galley horizontal constraint is -1");
+        goto REJECT;
+      }
+    }
+    else SetConstraint(c, MAX_LEN, MAX_LEN, MAX_LEN);  /* actually unused */
+
     debug1(DGA, DDD, "  expanding %s", EchoObject(target));
     tmp = CopyObject(target, no_fpos);
     Link(target_galley, tmp);
-    if( !FitsConstraint(0, 0, c) )
-    { debug0(DGA, D, "  reject: target_galley horizontal constraint is -1");
-      goto REJECT;
-    }
     env = DetachEnv(tmp);
-    SizeGalley(target_galley, env, external(target), threaded(target),
+    debug4(DGM, D, "  external_ver(%s) = %s, external_hor(%s) = %s",
+      SymName(actual(target)), bool(external_ver(target)),
+      SymName(actual(target)), bool(external_hor(target)));
+    SizeGalley(target_galley, env,
+	external_ver(target) || external_hor(target),
+	threaded(target),
 	non_blocking(target_index), trigger_externs(target_index),
 	&save_style(target), &c, whereto(hd), &dest_index, &recs, &tg_inners);
     debug1(DGA, DD, "  SizeGalley tg_inners: %s", DebugInnersNames(tg_inners));
@@ -335,15 +363,20 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
     dest = actual(dest_index);
 
     /* verify that hd satisfies any horizontal constraint on dest */
-    debug1(DGA, DDD, "  checking COL fit of hd in %s", SymName(actual(dest)));
-    Constrained(dest, &c, COL);
-    debug2(DSC, DD, "Constrained( %s, COL ) = %s",
-	EchoObject(dest), EchoConstraint(&c));
-    assert( constrained(c), "AttachGalley: dest unconstrained!" );
-    if( !FitsConstraint(0, 0, c) )
-    { debug0(DGA, D, "  reject: hd horizontal constraint is -1");
-      goto REJECT;
+    if( dim == ROWM )
+    {
+      debug1(DGA, DDD, "  checking hor fit of hd in %s",SymName(actual(dest)));
+      Constrained(dest, &c, 1-dim, &junk);
+      debug3(DSC, DD, "Constrained( %s, %s ) = %s",
+	EchoObject(dest), dimen(1-dim), EchoConstraint(&c));
+      assert( constrained(c), "AttachGalley: dest unconstrained!" );
+      if( !FitsConstraint(0, 0, c) )
+      { debug0(DGA, D, "  reject: hd horizontal constraint is -1");
+        goto REJECT;
+      }
     }
+
+    /* manifest and size the galley if not done yet */
     if( !sized(hd) )
     {
       debug2(DYY, D, "[ EnterErrorBlock(TRUE) (sizing galley %s into %s)",
@@ -352,17 +385,19 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
       n1 = nilobj;
       Child(y, Down(hd));
       env = DetachEnv(y);
-      /*** to set non_blocking() to FALSE seems doubtful!
-      SizeGalley(hd, env, TRUE, threaded(dest), FALSE, TRUE,
-		&save_style(dest), &c, nilobj, &n1, &recs, &hd_inners);
-      *** */
+      /*** threaded() only defined in ROWM case
       SizeGalley(hd, env, TRUE, threaded(dest), non_blocking(target_index),
 	TRUE, &save_style(dest), &c, nilobj, &n1, &recs, &hd_inners);
+      *** */
+      SizeGalley(hd, env, TRUE, dim == ROWM ? threaded(dest) : FALSE,
+	non_blocking(target_index), TRUE, &save_style(dest), &c, nilobj,
+	&n1, &recs, &hd_inners);
       debug1(DGA,DD,"  SizeGalley hd_inners: %s", DebugInnersNames(hd_inners));
       if( recs != nilobj )  ExpandRecursives(recs);
       if( need_precedes )		/* need an ordering constraint */
-      {	OBJECT index1 = New(PRECEDES);
-	OBJECT index2 = New(FOLLOWS);
+      {	OBJECT index1, index2;
+        New(index1, PRECEDES);
+	New(index2, FOLLOWS);
 	blocked(index2) = FALSE;
 	tmp = MakeWord(WORD, STR_EMPTY, no_fpos);
 	Link(index1, tmp);  Link(index2, tmp);
@@ -373,13 +408,16 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
       LeaveErrorBlock(TRUE);
       debug0(DYY, D, "] LeaveErrorBlock(TRUE) (finished sizing galley)");
     }
-    if( !FitsConstraint(back(hd, COL), fwd(hd, COL), c) )
-    { debug3(DGA, D, "  reject: hd %s,%s does not fit target_galley %s",
-	EchoLength(back(hd, COL)), EchoLength(fwd(hd, COL)),
-	EchoConstraint(&c));
-      Error(19, 3, "too little horizontal space for galley %s at %s",
-	WARN, &fpos(hd), SymName(actual(hd)), SymName(actual(dest)));
-      goto REJECT;
+
+    if( dim == ROWM )
+    { if( !FitsConstraint(back(hd, 1-dim), fwd(hd, 1-dim), c) )
+      { debug3(DGA, D, "  reject: hd %s,%s does not fit target_galley %s",
+	  EchoLength(back(hd, 1-dim)), EchoLength(fwd(hd, 1-dim)),
+	  EchoConstraint(&c));
+        Error(19, 3, "too little horizontal space for galley %s at %s",
+	  WARN, &fpos(hd), SymName(actual(hd)), SymName(actual(dest)));
+        goto REJECT;
+      }
     }
 
     /* check status of first component of hd */
@@ -389,14 +427,16 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
     {
       Child(y, link);
       debug1(DGA, DDD, "  examining %s", EchoIndex(y));
-      if( type(y) == SPLIT )  Child(y, DownDim(y, ROW));
+      if( type(y) == SPLIT )  Child(y, DownDim(y, dim));
       switch( type(y) )
       {
 
 	case EXPAND_IND:
 	case SCALE_IND:
+	case COVER_IND:
 	case GALL_PREC:
 	case GALL_FOLL:
+	case GALL_FOLL_OR_PREC:
 	case GALL_TARG:
 	case CROSS_PREC:
 	case CROSS_FOLL:
@@ -411,7 +451,7 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
 	    
 	  if( was_sized )
 	  { /* SizeGalley was not called, so hd_inners was not set by it */
-	    if( hd_inners == nilobj )  hd_inners = New(ACAT);
+	    if( hd_inners == nilobj )  New(hd_inners, ACAT);
 	    Link(hd_inners, y);
 	  }
 	  break;
@@ -419,30 +459,11 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
 
 	case RECEPTIVE:
 
-	  /* ***
-	  if( non_blocking(y) )
-	  { link = PrevDown(link);
-	    DeleteNode(y);
-	  }
-	  else
-	  *** */
 	  goto SUSPEND;
 
 
 	case RECEIVING:
 	    
-	  /* ***
-	  if( non_blocking(y) )
-	  { while( Down(y) != y )
-	    { Child(z, Down(y));
-	      DetachGalley(z);
-	      KillGalley(z);
-	    }
-	    link = PrevDown(link);
-	    DeleteNode(y);
-	  }
-	  else
-	  *** */
 	  goto SUSPEND;
 
 
@@ -499,13 +520,12 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
 	case VSHIFT:
 	case HSCALE:
 	case VSCALE:
+	case HCOVER:
+	case VCOVER:
 	case HCONTRACT:
 	case VCONTRACT:
 	case HEXPAND:
 	case VEXPAND:
-	case PADJUST:
-	case HADJUST:
-	case VADJUST:
 	case ROTATE:
 	case SCALE:
 	case INCGRAPHIC:
@@ -513,137 +533,290 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
 	case GRAPHIC:
 	case ACAT:
 	case HCAT:
+	case VCAT:
 	case ROW_THR:
+	case COL_THR:
 	    
-	  /* make sure y is not joined to a target below */
-	  for( zlink = NextDown(link);  zlink != hd;  zlink = NextDown(zlink) )
-	  { Child(z, zlink);
-	    switch( type(z) )
+
+	  if( dim == ROWM )
+	  {
+	    /* make sure y is not joined to a target below (vertical only) */
+	    for( zlink = NextDown(link); zlink != hd; zlink = NextDown(zlink) )
+	    { Child(z, zlink);
+	      switch( type(z) )
+	      {
+	        case RECEPTIVE:
+		
+		  if( non_blocking(z) )
+		  { zlink = PrevDown(zlink);
+		    DeleteNode(z);
+		  }
+		  else
+		  { y = z;
+		    goto SUSPEND;
+		  }
+		  break;
+
+
+	        case RECEIVING:
+		
+		  if( non_blocking(z) )
+		  { zlink = PrevDown(zlink);
+		    while( Down(z) != z )
+		    { Child(tmp, Down(y));
+		      if( opt_components(tmp) != nilobj )
+		      { DisposeObject(opt_components(tmp));
+		        opt_components(tmp) = nilobj;
+		        debug3(DOG, D, "AttachGalley(%s) de-optimizing %s %s",
+			  SymName(actual(hd)), SymName(actual(tmp)), "(join)");
+		      }
+		      DetachGalley(tmp);
+		      KillGalley(tmp, FALSE);
+		    }
+		    DeleteNode(z);
+		  }
+		  else
+		  { y = z;
+		    goto SUSPEND;
+		  }
+		  break;
+
+
+	        case GAP_OBJ:
+		
+		  if( !join(gap(z)) )  zlink = PrevDown(hd);
+		  break;
+
+
+	        default:	break;
+	      }
+	    }
+
+	    /* if HCAT, try vertical hyphenation (vertical galleys only) */
+	    if( type(y) == HCAT )  VerticalHyphenate(y);
+	  }
+
+
+	  /* check availability of parallel space for the first component */
+	  why = nilobj;
+	  Constrained(dest, &c, dim, &why);
+	  debug3(DGF, DD, "  dest parallel Constrained(%s, %s) = %s",
+	    EchoObject(dest), dimen(dim), EchoConstraint(&c));
+	  if( !FitsConstraint(back(y, dim), fwd(y, dim), c) )
+	  { BOOLEAN scaled;
+
+	    /* if forcing galley doesn't fit, try scaling first component */
+	    scaled = FALSE;
+	    if( actual(hd) != nilobj && force_target(actual(hd)) &&
+		size(y, dim) > 0 )
+	    { int scale_factor;
+	      scale_factor = ScaleToConstraint(back(y,dim), fwd(y,dim), &c);
+	      if( scale_factor > 0.5 * SF )
+	      {	char num1[20], num2[20];
+		sprintf(num1, "%.1fc", (float) size(y, dim) / CM);
+		sprintf(num2, "%.1fc", (float) bfc(c) / CM);
+		if( dim == ROWM )
+		  Error(19, 4, "%s object too high for %s space; %s inserted",
+		    WARN, &fpos(y), num1, num2, KW_SCALE);
+		else
+		  Error(19, 5, "%s object too wide for %s space; %s inserted",
+		    WARN, &fpos(y), num1, num2, KW_SCALE);
+		y = InterposeScale(y, scale_factor, dim);
+		scaled = TRUE;
+	      }
+	    }
+
+	    /* otherwise we must reject, and warn the user */
+	    if( !scaled )
 	    {
-	      case RECEPTIVE:	if( non_blocking(z) )
-				{ zlink = PrevDown(zlink);
-				  DeleteNode(z);
-				}
-				else
-				{ y = z;
-				  goto SUSPEND;
-				}
-				break;
+	      debug3(DGA, D, "  reject: vsize %s,%s in %s; y=",
+		EchoLength(back(y, dim)), EchoLength(fwd(y, dim)),
+		EchoConstraint(&c));
+	      ifdebug(DGA, D, DebugObject(y));
+	      goto REJECT;
+	    }
 
-	      case RECEIVING:	if( non_blocking(z) )
-				{ zlink = PrevDown(zlink);
-				  while( Down(z) != z )
-				  { Child(tmp, Down(y));
-				    DetachGalley(tmp);
-				    KillGalley(tmp);
-				  }
-				  DeleteNode(z);
-				}
-				else
-				{ y = z;
-				  goto SUSPEND;
-				}
-				break;
+	  }
 
-	      case GAP_OBJ:	if( !join(gap(z)) )  zlink = PrevDown(hd);
-				break;
+	  /* check availability of perpendicular space for first component */
+	  if( dim == ROWM )
+	  { perp_back = back(hd, 1-dim);  perp_fwd = fwd(hd, 1-dim);
+	  }
+	  else
+	  { perp_back = back(y, 1-dim);  perp_fwd = fwd(y, 1-dim);
+	  }
+	  Constrained(dest, &c, 1-dim, &junk);
+	  debug3(DGF, DD, "  dest perpendicular Constrained(%s, %s) = %s",
+	    EchoObject(dest), dimen(1-dim), EchoConstraint(&c));
+	  if( !FitsConstraint(perp_back, perp_fwd, c) )
+	  { BOOLEAN scaled;
 
-	      default:		break;
+	    /* if forcing galley doesn't fit, try scaling first component */
+	    scaled = FALSE;
+	    if( actual(hd) != nilobj && force_target(actual(hd)) &&
+		perp_back + perp_fwd > 0 )
+	    { int scale_factor;
+	      scale_factor = ScaleToConstraint(perp_back, perp_fwd, &c);
+	      if( scale_factor > 0.5 * SF )
+	      {	char num1[20], num2[20];
+		sprintf(num1, "%.1fc", (float) (perp_back + perp_fwd) / CM);
+		sprintf(num2, "%.1fc", (float) bfc(c) / CM);
+		if( 1-dim == ROWM )
+		  Error(19, 6, "%s object too high for %s space; %s inserted",
+		    WARN, &fpos(y), num1, num2, KW_SCALE);
+		else
+		  Error(19, 7, "%s object too wide for %s space; %s inserted",
+		    WARN, &fpos(y), num1, num2, KW_SCALE);
+		y = InterposeScale(y, scale_factor, 1-dim);
+		scaled = TRUE;
+	      }
+	    }
+
+	    /* otherwise we must reject, and warn the user */
+	    if( !scaled )
+	    {
+	      debug3(DGA, D, "  reject: vsize %s,%s in %s; y=",
+		EchoLength(perp_back), EchoLength(perp_fwd),
+		EchoConstraint(&c));
+	      ifdebug(DGA, D, DebugObject(y));
+	      goto REJECT;
+	    }
+
+	  }
+
+	  /* dest seems OK, so perform its size adjustments */
+	  debug0(DSA, D, "calling AdjustSize from AttachGalley (a)");
+	  AdjustSize(dest, back(y, dim), fwd(y, dim), dim);
+	  debug0(DSA, D, "calling AdjustSize from AttachGalley (b)");
+	  AdjustSize(dest, perp_back, perp_fwd, 1-dim);
+
+
+	  /* now check parallel space for target_galley in target */
+	  Constrained(target, &c, dim, &why);
+	  debug3(DGF, DD, "  target parallel Constrained(%s, %s) = %s",
+	    EchoObject(target), dimen(dim), EchoConstraint(&c));
+	  Child(z, LastDown(target_galley));  /* works in all cases? */
+	  assert( !is_index(type(z)), "AttachGalley: is_index(z)!" );
+	  assert( back(z, dim)>=0 && fwd(z, dim)>=0, "AttachGalley: z size!" );
+	  if( !FitsConstraint(back(z, dim), fwd(z, dim), c) )
+	  { BOOLEAN scaled;
+
+	    debug2(DGA, D, "  why     = %d %s", (int) why, EchoObject(why));
+	    debug2(DGA, D, "  limiter = %d %s", (int) limiter(hd),
+	      EchoObject(limiter(hd)));
+
+	    /* if forcing galley doesn't fit, try scaling z */
+	    scaled = FALSE;
+	    if( actual(hd) != nilobj && force_target(actual(hd)) &&
+		size(z, dim) > 0 && limiter(hd) != why )
+	    { int scale_factor;
+	      scale_factor = ScaleToConstraint(back(z,dim), fwd(z,dim), &c);
+	      if( scale_factor > 0.5 * SF )
+	      {	char num1[20], num2[20];
+		sprintf(num1, "%.1fc", (float) size(z, dim) / CM);
+		sprintf(num2, "%.1fc", (float) bfc(c) / CM);
+		if( dim == ROWM )
+		  Error(19, 8, "%s object too high for %s space; %s inserted",
+		    WARN, &fpos(y), num1, num2, KW_SCALE);
+		else
+		  Error(19, 9, "%s object too wide for %s space; %s inserted",
+		    WARN, &fpos(y), num1, num2, KW_SCALE);
+		z = InterposeWideOrHigh(z, dim);
+		z = InterposeScale(z, scale_factor, dim);
+		scaled = TRUE;
+	      }
+	    }
+
+	    if( !scaled )
+	    {
+	      limiter(hd) = why;
+	      debug3(DGA, D, "  set limiter(%s) = %d %s", SymName(actual(hd)),
+		(int) limiter(hd), EchoObject(limiter(hd)));
+	      debug3(DGA, D, "  reject: size was %s,%s in %s; y =",
+		EchoLength(back(z, dim)), EchoLength(fwd(z, dim)),
+		EchoConstraint(&c));
+	      ifdebug(DGA, D, DebugObject(y));
+	      goto REJECT;
+	    }
+	  }
+	  limiter(hd) = why;
+	  debug3(DGA, D, "  set limiter(%s) = %d %s", SymName(actual(hd)),
+	    (int) limiter(hd), EchoObject(limiter(hd)));
+
+	  /* now check perpendicular space for target_galley in target */
+	  Constrained(target, &c, 1-dim, &junk);
+	  debug3(DGF, DD, "  target perpendicular Constrained(%s, %s) = %s",
+	    EchoObject(target), dimen(1-dim), EchoConstraint(&c));
+	  Child(z, LastDown(target_galley));  /* works in all cases? */
+	  assert( !is_index(type(z)), "AttachGalley: is_index(z)!" );
+	  assert( back(z, 1-dim)>=0 && fwd(z, 1-dim)>=0,
+	    "AttachGalley: z size (perpendicular)!" );
+	  if( !FitsConstraint(back(z, 1-dim), fwd(z, 1-dim), c) )
+	  { BOOLEAN scaled;
+
+	    /* if forcing galley doesn't fit, try scaling z */
+	    scaled = FALSE;
+	    if( actual(hd) != nilobj &&
+		force_target(actual(hd)) && size(z, 1-dim) > 0 )
+	    { int scale_factor;
+	      scale_factor = ScaleToConstraint(back(z,1-dim), fwd(z,1-dim), &c);
+	      if( scale_factor > 0.5 * SF )
+	      {	char num1[20], num2[20];
+		sprintf(num1, "%.1fc", (float) size(z, 1-dim) / CM);
+		sprintf(num2, "%.1fc", (float) bfc(c) / CM);
+		if( 1-dim == ROWM )
+		  Error(19, 10, "%s object too high for %s space; %s inserted",
+		    WARN, &fpos(y), num1, num2, KW_SCALE);
+		else
+		  Error(19, 11, "%s object too wide for %s space; %s inserted",
+		    WARN, &fpos(y), num1, num2, KW_SCALE);
+		z = InterposeWideOrHigh(z, 1-dim);
+		z = InterposeScale(z, scale_factor, 1-dim);
+		scaled = TRUE;
+	      }
+	    }
+
+	    if( !scaled )
+	    {
+	      debug3(DGA, D, "  reject: size was %s,%s in %s; y =",
+		EchoLength(back(z, 1-dim)), EchoLength(fwd(z, 1-dim)),
+		EchoConstraint(&c));
+	      ifdebug(DGA, D, DebugObject(y));
+	      goto REJECT;
 	    }
 	  }
 
-	  /* if HCAT, try vertical hyphenation before doing anything else */
-	  if( type(y) == HCAT )  VerticalHyphenate(y);
-
-	  /* check availability of vertical space for the first component */
-	  if( !external(dest) )
-	  { Constrained(dest, &c, ROW);
-	    debug2(DSC, DD, "Constrained( %s, ROW ) = %s",
-	      EchoObject(dest), EchoConstraint(&c));
-	    if( !FitsConstraint(back(y, ROW), fwd(y, ROW), c) )
-	    { BOOLEAN scaled;
-
-	      /* if forcing galley doesn't fit, try scaling first component */
-	      scaled = FALSE;
-	      if( actual(hd) != nilobj && force_target(actual(hd))
-		 && size(y, ROW) > 0)
-	      { int scale_factor;
-		scale_factor = ScaleToConstraint(back(y,ROW), fwd(y,ROW), &c);
-		/* scale_factor = (bfc(c) * SF) / size(y, ROW); */
-		if( scale_factor > 0.5 * SF )
-		{ char num1[20], num2[20];
-		  sprintf(num1, "%.1fc", (float) size(y, ROW) / CM);
-		  sprintf(num2, "%.1fc", (float) bfc(c) / CM);
-		  Error(19, 7, "%s object too high for %s space; %s inserted",
-		    WARN, &fpos(y), num1, num2, KW_VSCALE);
-		  y = InterposeVScale(y, scale_factor);
-		  scaled = TRUE;
-		}
-	      }
-
-	      /* otherwise we must reject, and warn the user */
-	      if( !scaled )
-	      { Error(19, 4, "this component of %s did not fit into its nearest target",
-		  WARN, &fpos(y), SymName(actual(hd)));
-	        debug3(DGA, D, "  reject: vsize %s,%s in %s; y=",
-		  EchoLength(back(y, ROW)), EchoLength(fwd(y, ROW)),
-		  EchoConstraint(&c));
-	        ifdebug(DGA, D, DebugObject(y));
-	        goto REJECT;
-	      }
-	    }
-	    debug0(DSA, D, "calling AdjustSize from AttachGalley (a)");
-	    AdjustSize(dest, back(y, ROW), fwd(y, ROW), ROW);
+	  /* target seems OK, so adjust sizes and accept */
+	  if( external_hor(target) )
+	  {
+	    /* don't adjust any sizes, none to adjust */
+	    debug0(DSA, D, "not calling AdjustSize from AttachGalley (c)");
 	  }
-	  if( !external(target) )
-	  { Constrained(target, &c, ROW);
-	    debug2(DSC, DD, "Constrained( %s, ROW ) = %s",
-			EchoObject(target), EchoConstraint(&c));
+	  else if( external_ver(target) )
+	  {
+	    /* adjust perp size only, to galley size */
+	    debug0(DSA, D, "calling AdjustSize from AttachGalley (d)");
+	    AdjustSize(target, back(target_galley, 1-dim),
+	      fwd(target_galley, 1-dim), 1-dim);
+	  }
+	  else
+	  {
+	    /* adjust both directions, using z (last component) */
 	    Child(z, LastDown(target_galley));
-	    assert( !is_index(type(z)), "AttachGalley: is_index(z)!" );
-	    assert( back(z, ROW) >= 0 && fwd(z, ROW) >= 0,
-			"AttachGalley: negative z sizes!" );
-	    if( !FitsConstraint(back(z, ROW), fwd(z, ROW), c) )
-	    { BOOLEAN scaled;
-
-	      /* if forcing galley doesn't fit, try scaling z */
-	      scaled = FALSE;
-	      if( actual(hd) != nilobj && force_target(actual(hd))
-		 && size(z, ROW) > 0 )
-	      { int scale_factor;
-		/* scale_factor = (bfc(c) * SF) / size(z, ROW); */
-		scale_factor = ScaleToConstraint(back(z,ROW), fwd(z,ROW), &c);
-		if( scale_factor > 0.5 * SF )
-		{ char num1[20], num2[20];
-		  sprintf(num1, "%.1fc", (float) size(z, ROW) / CM);
-		  sprintf(num2, "%.1fc", (float) bfc(c) / CM);
-		  Error(19, 7, "%s object too high for %s space; %s inserted",
-		    WARN, &fpos(y), num1, num2, KW_VSCALE);
-		  z = InterposeHigh(z);
-		  z = InterposeVScale(z, scale_factor);
-		  scaled = TRUE;
-		}
-	      }
-
-	      if( !scaled )
-	      { Error(19, 5, "this component of %s did not fit into its nearest target",
-	          WARN, &fpos(y), SymName(actual(hd)));
-	        debug3(DGA, D, "  reject: size was %s,%s in %s; y =",
-		  EchoLength(back(z, ROW)), EchoLength(fwd(z, ROW)),
-		  EchoConstraint(&c));
-	        ifdebug(DGA, D, DebugObject(y));
-	        goto REJECT;
-	      }
-	    }
-	    debug0(DSA, D, "calling AdjustSize from AttachGalley (b)");
-	    AdjustSize(target, back(z, ROW), fwd(z, ROW), ROW);
+	    debug0(DSA, D, "AttachGalley AdjustSize using z =");
+	    ifdebug(DSA, D, DebugObject(z));
+	    debug0(DSA, D, "calling AdjustSize from AttachGalley (e)");
+	    AdjustSize(target, back(z, dim), fwd(z, dim), dim);
+	    debug0(DSA, D, "calling AdjustSize from AttachGalley (f)");
+	    AdjustSize(target, back(z, 1-dim), fwd(z, 1-dim), 1-dim);
 	  }
+
 	  goto ACCEPT;
 
 
 	default:
 	    
-	  Error(19, 6, "AttachGalley: %s", INTERN, &fpos(y), Image(type(y)));
+	  assert1(FALSE, "AttachGalley:", Image(type(y)));
 	  break;
 
       } /* end switch */
@@ -701,7 +874,7 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
       debug0(DYY, D, "] LeaveErrorBlock(TRUE) (REJECT)");
       if( tg_inners != nilobj )  DisposeObject(tg_inners), tg_inners = nilobj;
       DisposeObject(target_galley);
-      if( backward(hd) && !sized(hd) )
+      if( foll_or_prec(hd) == GALL_PREC && !sized(hd) )
       {
 	/* move to just before the failed target */
 	MoveLink(Up(hd_index), Up(target_index), PARENT);
@@ -741,17 +914,11 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
     ACCEPT:
 	
       /* accept first component; now committed to the attach */
-      debug1(DGA, D, "  accept %s", EchoObject(y));
+      debug3(DGA, D, "  accept %s %s %s", Image(type(y)), EchoObject(y),
+	EchoFilePos(&fpos(y)));
       LeaveErrorBlock(TRUE);
       debug0(DYY, D, "] LeaveErrorBlock(TRUE) (ACCEPT)");
 
-      /* adjust horizontal sizes */
-      debug0(DSA, D, "calling AdjustSize from AttachGalley (c)");
-      AdjustSize(dest, back(hd, COL), fwd(hd, COL), COL);
-      debug0(DSA, D, "calling AdjustSize from AttachGalley (d)");
-      AdjustSize(target, back(target_galley, COL),
-				fwd(target_galley, COL), COL);
-		
       /* attach hd to dest */
       MoveLink(Up(hd), dest_index, PARENT);
       assert( type(hd_index) == UNATTACHED, "AttachGalley: type(hd_index)!" );
@@ -759,12 +926,20 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
 
       /* move first component of hd into dest */
       /* nb Interpose must be done after all AdjustSize calls */
-      if( !external(dest) )   Interpose(dest, VCAT, hd, y);
+      if( dim == ROWM && !external_ver(dest) )
+	Interpose(dest, VCAT, hd, y);
+      else if( dim == COLM && !external_hor(dest) )
+      { Interpose(dest, ACAT, y, y);
+	Parent(junk, Up(dest));
+	assert( type(junk) == ACAT, "AttachGalley: type(junk) != ACAT!" );
+	StyleCopy(save_style(junk), save_style(dest));
+	adjust_cat(junk) = padjust(save_style(junk));
+      }
       Promote(hd, link == hd ? hd : NextDown(link), dest_index);
 
       /* move target_galley into target */
       /* nb Interpose must be done after all AdjustSize calls */
-      if( !external(target) )
+      if( !(external_ver(target) || external_hor(target)) )
       {	Child(z, LastDown(target_galley));
 	Interpose(target, VCAT, z, z);
       }
@@ -785,6 +960,14 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
 	*inners = tg_inners;
       }
       debug0(DGA, D, "] AttachGalley returning ATTACH_ACCEPT");
+      ifdebug(DGA, D,
+	if( dim == COLM && !external_hor(dest) )
+	{ OBJECT z;
+	  Parent(z, Up(dest));
+	  debug2(DGA, D, "  COLM dest_encl on exit = %s %s",
+	    Image(type(z)), EchoObject(z));
+	}
+      )
       return ATTACH_ACCEPT;
 
   } /* end for */
