@@ -1,9 +1,9 @@
 /*@z22.c:Galley Service:Interpose()@******************************************/
 /*                                                                           */
-/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.06)                       */
-/*  COPYRIGHT (C) 1994 Jeffrey H. Kingston                                   */
+/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.08)                       */
+/*  COPYRIGHT (C) 1991, 1996 Jeffrey H. Kingston                             */
 /*                                                                           */
-/*  Jeffrey H. Kingston (jeff@cs.su.oz.au)                                   */
+/*  Jeffrey H. Kingston (jeff@cs.usyd.edu.au)                                */
 /*  Basser Department of Computer Science                                    */
 /*  The University of Sydney 2006                                            */
 /*  AUSTRALIA                                                                */
@@ -30,8 +30,6 @@
 /*                                                                           */
 /*****************************************************************************/
 #include "externs"
-#define	LAST_ADJUST	1
-#define	ALL_ADJUST	2
 
 
 /*****************************************************************************/
@@ -44,13 +42,14 @@
 /*****************************************************************************/
 
 void Interpose(OBJECT z, int typ, OBJECT x, OBJECT y)
-{ OBJECT encl = New(typ);
+{ OBJECT encl;
+  New(encl, typ);
   FposCopy(fpos(encl), fpos(y));
   ReplaceNode(encl, z);  Link(encl, z);
-  back(encl, COL) = back(x, COL);
-  fwd(encl, COL) = fwd(x, COL);
-  back(encl, ROW) = back(y, ROW);
-  fwd(encl, ROW) = fwd(y, ROW);
+  back(encl, COLM) = back(x, COLM);
+  fwd(encl, COLM) = fwd(x, COLM);
+  back(encl, ROWM) = back(y, ROWM);
+  fwd(encl, ROWM) = fwd(y, ROWM);
 } /* end Interpose */
 
 
@@ -126,7 +125,7 @@ void FlushInners(OBJECT inners, OBJECT hd)
 
       default:
       
-	Error(22, 1, "FlushInners: %s", INTERN, &fpos(y), Image(type(y)));
+	assert1(FALSE, "FlushInners:", Image(type(y)));
 	break;
     }
   }
@@ -144,8 +143,8 @@ void FlushInners(OBJECT inners, OBJECT hd)
 
 void ExpandRecursives(OBJECT recs)
 { CONSTRAINT non_c, hc, vc;
-  OBJECT target_index, target, z, n1, inners, newrecs, hd, tmp, env;
-  debug0(DCR, D, "ExpandRecursives(recs)");
+  OBJECT target_index, target, z, n1, inners, newrecs, hd, tmp, env, why;
+  debug0(DCR, DD, "ExpandRecursives(recs)");
   SetConstraint(non_c, MAX_LEN, MAX_LEN, MAX_LEN);  n1 = nilobj;
   assert(recs != nilobj, "ExpandRecursives: recs == nilobj!");
   while( Down(recs) != recs )
@@ -156,20 +155,25 @@ void ExpandRecursives(OBJECT recs)
       EchoObject(target));
 
     /* expand body of target, convert to galley, and check size */
-    hd = New(HEAD);  actual(hd) = actual(target);  must_expand(hd) = TRUE;
-    whereto(hd) = ready_galls(hd) = nilobj;  backward(hd) = sized(hd) = FALSE;
+    New(hd, HEAD);  actual(hd) = actual(target);  must_expand(hd) = TRUE;
+    limiter(hd) = opt_components(hd) = opt_constraints(hd) = nilobj;
+    gall_dir(hd) = horiz_galley(actual(target));
+    whereto(hd) = ready_galls(hd) = nilobj;
+    foll_or_prec(hd) = GALL_FOLL;
+    sized(hd) = FALSE;
     tmp =  CopyObject(target, &fpos(target));  env = DetachEnv(tmp);
     Link(hd, tmp);  Link(target_index, hd);
-    SizeGalley(hd, env, external(target), threaded(target), FALSE, FALSE,
+    SizeGalley(hd, env, external_ver(target),
+      gall_dir(hd) == ROWM ? threaded(target) : FALSE, FALSE, FALSE,
       &save_style(target), &non_c, nilobj, &n1, &newrecs, &inners);
     debug0(DCR, DDD, "    as galley:");
     ifdebug(DCR, DDD, DebugObject(hd));
-    Constrained(target, &hc, COL);
-    debug2(DSC, DD, "Constrained( %s, COL ) = %s",
+    Constrained(target, &hc, COLM, &why);
+    debug2(DSC, DD, "Constrained( %s, COLM ) = %s",
       EchoObject(target), EchoConstraint(&hc));
     debug3(DCR, DD, "    horizontal size: (%s, %s); constraint: %s",
-      EchoLength(back(hd, COL)), EchoLength(fwd(hd, COL)), EchoConstraint(&hc));
-    if( !FitsConstraint(back(hd, COL), fwd(hd, COL), hc) )
+      EchoLength(back(hd, COLM)), EchoLength(fwd(hd, COLM)), EchoConstraint(&hc));
+    if( !FitsConstraint(back(hd, COLM), fwd(hd, COLM), hc) )
     { DisposeChild(Up(hd));
       if( inners != nilobj ) DisposeObject(inners);
       if( newrecs != nilobj ) DisposeObject(newrecs);
@@ -177,14 +181,14 @@ void ExpandRecursives(OBJECT recs)
       debug0(DCR, DD, "    rejecting (too wide)");
       continue;
     }
-    if( !external(target) )
-    { Constrained(target, &vc, ROW);
-      debug2(DSC, DD, "Constrained( %s, ROW ) = %s",
+    if( !external_ver(target) )
+    { Constrained(target, &vc, ROWM, &why);
+      debug2(DSC, DD, "Constrained( %s, ROWM ) = %s",
 	EchoObject(target), EchoConstraint(&vc));
       Child(z, LastDown(hd));
       debug3(DCR, DD, "    vsize: (%s, %s); constraint: %s",
-	EchoLength(back(z, ROW)), EchoLength(fwd(z, ROW)), EchoConstraint(&vc));
-      if( !FitsConstraint(back(z, ROW), fwd(z, ROW), vc) )
+	EchoLength(back(z, ROWM)), EchoLength(fwd(z, ROWM)), EchoConstraint(&vc));
+      if( !FitsConstraint(back(z, ROWM), fwd(z, ROWM), vc) )
       {	DisposeChild(Up(hd));
 	if( inners != nilobj ) DisposeObject(inners);
 	if( newrecs != nilobj ) DisposeObject(newrecs);
@@ -196,10 +200,10 @@ void ExpandRecursives(OBJECT recs)
 
     /* object fits; adjust sizes and promote */
     debug0(DSA, D, "calling AdjustSize from ExpandRecursives (a)");
-    AdjustSize(target, back(hd, COL), fwd(hd, COL), COL);
-    if( !external(target) )
+    AdjustSize(target, back(hd, COLM), fwd(hd, COLM), COLM);
+    if( !external_ver(target) )
     { debug0(DSA, D, "calling AdjustSize from ExpandRecursives (b)");
-      AdjustSize(target, back(z, ROW), fwd(z, ROW), ROW);
+      AdjustSize(target, back(z, ROWM), fwd(z, ROWM), ROWM);
       Interpose(target, VCAT, z, z);
     }
     Promote(hd, hd, target_index);  DeleteNode(hd);
@@ -208,7 +212,7 @@ void ExpandRecursives(OBJECT recs)
     if( newrecs != nilobj )  MergeNode(recs, newrecs);
   } /* end while */
   Dispose(recs);
-  debug0(DCR, D, "ExpandRecursives returning.");
+  debug0(DCR, DD, "ExpandRecursives returning.");
 } /* end ExpandRecursives */
 
 /*@::FindSplitInGalley()@*****************************************************/
@@ -229,7 +233,7 @@ static OBJECT FindSplitInGalley(OBJECT hd)
   if( link == hd )
   { debug0(DGF, D, "FindSplitInGalley failing, no definite component; hd =");
     ifdebug(DGF, D, DebugObject(hd));
-    Error(22, 2, "FindSplit: missing galley component", INTERN, &fpos(hd));
+    Error(22, 1, "FindSplit: missing galley component", INTERN, &fpos(hd));
   }
   while( type(y) != SPLIT )  switch( type(y) )
   {
@@ -241,8 +245,6 @@ static OBJECT FindSplitInGalley(OBJECT hd)
     case VSHIFT:
     case VCONTRACT:
     case VEXPAND:
-    case PADJUST:
-    case VADJUST:
 
       Child(y, Down(y));
       break;
@@ -261,9 +263,10 @@ static OBJECT FindSplitInGalley(OBJECT hd)
     case SCALE:
     case HSCALE:
     case VSCALE:
+    case HCOVER:
+    case VCOVER:
     case HCONTRACT:
     case HEXPAND:
-    case HADJUST:
     case ROTATE:
     case INCGRAPHIC:
     case SINCGRAPHIC:
@@ -271,13 +274,13 @@ static OBJECT FindSplitInGalley(OBJECT hd)
 
       debug0(DGF, D, "FindSplitInGalley(hd) failing, hd =");
       ifdebug(DGF, D, DebugObject(hd));
-      Error(22, 3, "FindSplitInGalley failed", INTERN, &fpos(y),Image(type(y)));
+      Error(22, 2, "FindSplitInGalley failed", INTERN, &fpos(y),Image(type(y)));
       break;
 
 
     default:
     
-      Error(22, 4, "FindSplitInGalley: %s", INTERN, &fpos(y), Image(type(y)));
+      assert1(FALSE, "FindSplitInGalley:", Image(type(y)));
       break;
 
   }
@@ -287,69 +290,158 @@ static OBJECT FindSplitInGalley(OBJECT hd)
 
 /*@::Promote()@***************************************************************/
 /*                                                                           */
-/*  Promote(x, stop_link, dest_index)                                        */
+/*  Promote(hd, stop_link, dest_index)                                       */
 /*                                                                           */
-/*  Promote components of galley x into its destination (dest), up to but    */
-/*  not including the one linked to x by link stop_link, which always        */
+/*  Promote components of galley hd into its destination (dest), up to but   */
+/*  not including the one linked to hd by link stop_link, which always       */
 /*  follows a component.  No size adjustments are made, except that when     */
-/*  two col_thr nodes are merged, a COL adjustment is made to the result.    */
+/*  two col_thr nodes are merged, a COLM adjustment is made to the result.   */
 /*                                                                           */
 /*****************************************************************************/
 
-void Promote(OBJECT x, OBJECT stop_link, OBJECT dest_index)
+void Promote(OBJECT hd, OBJECT stop_link, OBJECT dest_index)
 {
   /* these two variables refer to the root galley only */
   static BOOLEAN first = TRUE;	/* TRUE when the first component not written */
   static OBJECT page_label=nilobj;	/* current page label object         */
 
-  OBJECT dest, link, y, z, tmp1, tmp2;  FULL_CHAR *label_string;
+  OBJECT dest, link, y, z, tmp1, tmp2, why;  FULL_CHAR *label_string;
   int dim;
-  debug1(DGS, D, "Promote(%s, stop_link)", SymName(actual(x)));
+  debug1(DGS, DD, "Promote(%s, stop_link)", SymName(actual(hd)));
 
-  assert( type(x) == HEAD, "Promote: x!" );
-  assert( type(stop_link) == LINK || stop_link == x, "Promote: stop_link!" );
-  assert( stop_link != Down(x), "Promote: stop_link == Down(x)!" );
+  assert( type(hd) == HEAD, "Promote: hd!" );
+  assert( type(stop_link) == LINK || stop_link == hd, "Promote: stop_link!" );
+  assert( stop_link != Down(hd), "Promote: stop_link == Down(hd)!" );
   type(dest_index) = RECEIVING;
   dest = actual(dest_index);
 
   /* insert final gap if galley is ending */
-  if( stop_link != x )
+  if( stop_link != hd )
   { Child(y, stop_link);
     assert( type(y) == GAP_OBJ, "Promote: missing GAP_OBJ!" );
     stop_link = NextDown(stop_link);
   }
   else
-  { y = New(GAP_OBJ);
-    FposCopy(fpos(y), fpos(x));
+  { New(y, GAP_OBJ);
+    FposCopy(fpos(y), fpos(hd));
     hspace(y) = 0;  vspace(y) = 1;
-    /* SetGap(gap(y), FALSE, seen_nojoin(x), FIXED_UNIT, NO_MODE, 0); */
+    /* SetGap(gap(y), FALSE, seen_nojoin(hd), FIXED_UNIT, NO_MODE, 0); */
     SetGap(gap(y), FALSE, TRUE, FIXED_UNIT, NO_MODE, 0);
     /* SetGap(gap(y), FALSE, threaded(dest), FIXED_UNIT, NO_MODE, 0); */
     /* ClearGap(gap(y)); */
     Link(stop_link, y);
   }
 
+  /* if optimizing, add to dummy paragraph containing components and gaps */
+  if( opt_components(hd) != nilobj )
+  { OBJECT last, tmp;
+
+    debug1(DOG, DD, "Promote(%s) optimizing:", SymName(actual(hd)));
+    if( LastDown(opt_components(hd))!=opt_components(hd) && !opt_gazumped(hd) )
+    {
+      Child(last, LastDown(opt_components(hd)));
+    }
+    else last = nilobj;
+    for( link = Down(hd);  link != stop_link;  link = NextDown(link) )
+    { Child(y, link);
+      if( type(y) == GAP_OBJ )
+      {
+	if( last == nilobj )
+	{
+	  /* do nothing, gap cannot separate definite objects */
+	  debug1(DOG, DD, "  skipping initial GAP_OBJ %s", EchoGap(&gap(y)));
+	}
+	else if( type(last) == GAP_OBJ )
+	{
+	  /* previous gap must have preceded an indefinite, so overwrite it */
+	  FposCopy(fpos(last), fpos(y));
+	  debug2(DOG, DD, "  overwriting GAP_OBJ %s with %s",
+	    EchoGap(&gap(last)), EchoGap(&gap(y)));
+	  GapCopy(gap(last), gap(y));
+	  if( Down(last) != last )  DisposeChild(Down(last));
+	  if( Down(y) != y )
+	  { Child(tmp, Down(y));
+	    tmp = CopyObject(tmp, no_fpos);
+	    Link(last, tmp);
+	  }
+	  join(gap(last)) = TRUE;  /* irrelevant but improves debug output */
+	}
+	else
+	{
+	  /* previous was definite, so this gap must be stored */
+	  opt_gazumped(hd) = FALSE;
+	  New(last, GAP_OBJ);
+	  FposCopy(fpos(last), fpos(y));
+	  GapCopy(gap(last), gap(y));
+	  join(gap(last)) = TRUE;  /* irrelevant but improves debug output */
+	  hspace(last) = 1;
+	  vspace(last) = 0;
+	  Link(opt_components(hd), last);
+	  debug1(DOG, DD, "  adding GAP_OBJ %s", EchoGap(&gap(last)));
+	}
+      }
+      else if( is_word(type(y)) )
+      {
+	/* definite, must be stored */
+	opt_gazumped(hd) = FALSE;
+	last = MakeWord(type(y), string(y), &fpos(y));
+	back(last, COLM) = back(y, gall_dir(hd));
+	fwd(last, COLM) = fwd(y, gall_dir(hd));
+	word_font(last) = word_font(y);
+	word_colour(last) = word_colour(y);
+	word_language(last) = word_language(y);
+	word_hyph(last) = word_hyph(y);
+	Link(opt_components(hd), last);
+	debug2(DOG, DD, "  adding %s \"%s\"", Image(type(last)), string(last));
+      }
+      else if( is_indefinite(type(y)) )
+      {
+	/* indefinite, always skip these */
+	/* ***
+	New(z, NULL_CLOS);
+	FposCopy(fpos(z), fpos(y));
+	back(z, COLM) = 0;
+	fwd(z, COLM) = 0;
+	Link(opt_components(hd), z);
+	debug1(DOG, DD, "  adding %s", KW_NULL);
+	*** */
+      }
+      else if( is_definite(type(y)) )
+      {
+	/* definite other than WORD, add it */
+	opt_gazumped(hd) = FALSE;
+	last = MakeWord(QWORD, AsciiToFull("w"), &fpos(y));
+	back(last, COLM) = back(y, gall_dir(hd));
+	fwd(last, COLM) = fwd(y, gall_dir(hd));
+	Link(opt_components(hd), last);
+	debug1(DOG, DD, "  adding word for %s", EchoObject(y));
+      }
+    }
+    debug1(DOG, DD, "Promote(%s) end optimizing", SymName(actual(hd)));
+  }
+
   /* error if promoting a seen_nojoin galley into a threaded destination */
-  if( seen_nojoin(x) && threaded(dest) )
-    Error(22, 5, "galley %s must have a single column mark",
-      FATAL, &fpos(x), SymName(actual(x)));
+  if( seen_nojoin(hd) && gall_dir(hd) == ROWM && threaded(dest) )
+    Error(22, 3, "galley %s must have a single column mark",
+      FATAL, &fpos(hd), SymName(actual(hd)));
 
   /* make nojoin status clear by adding an extra gap at start if needed */
-  if( !external(dest) && seen_nojoin(x) && join(gap(y)) )
+  if( gall_dir(hd) == ROWM && !external_ver(dest) && seen_nojoin(hd) &&
+      join(gap(y)) )
   { OBJECT prnt, extra_null, extra_gap;
 
     /* add nojoin gap at start */
     Parent(prnt, Up(dest));  /* can't be threaded */
     assert( type(prnt) == VCAT, "Promote: nojoin case, can't find VCAT" );
-    extra_null = New(NULL_CLOS);
-    back(extra_null, COL) = fwd(extra_null, COL) = 0;
-    back(extra_null, ROW) = fwd(extra_null, ROW) = 0;
-    extra_gap = New(GAP_OBJ);
+    New(extra_null, NULL_CLOS);
+    back(extra_null, COLM) = fwd(extra_null, COLM) = 0;
+    back(extra_null, ROWM) = fwd(extra_null, ROWM) = 0;
+    New(extra_gap, GAP_OBJ);
     hspace(extra_gap) = vspace(extra_gap) = 0;
     SetGap(gap(extra_gap), FALSE, FALSE, FIXED_UNIT, EDGE_MODE, 0);
     Link(Down(prnt), extra_gap);
     Link(Down(prnt), extra_null);
-    debug0(DGS, D, "  Promote adding extra nojoin gap");
+    debug0(DGS, DD, "  Promote adding extra nojoin gap");
     /* join(gap(y)) = FALSE; */
   }
 
@@ -357,15 +449,17 @@ void Promote(OBJECT x, OBJECT stop_link, OBJECT dest_index)
   /* if promoting out of root galley, do special things */
   if( actual(dest) == PrintSym )
   { CONSTRAINT c;
-    link = x;
+    link = hd;
     while( NextDown(link) != stop_link )
     { Child(y, NextDown(link));
-      debug1(DGS, D, "root promote %s", EchoObject(y));
-      if( type(y) == SPLIT )  Child(y, DownDim(y, ROW));
+      debug2(DGS, DD, "root promote %s %s", Image(type(y)),
+	is_definite(type(y)) ? STR_EMPTY : EchoObject(y));
+      if( type(y) == SPLIT )  Child(y, DownDim(y, ROWM));
       switch( type(y) )
       {
 
 	case SCALE_IND:
+	case COVER_IND:
 	case PRECEDES:
       
 	  DisposeChild(NextDown(link));
@@ -382,9 +476,9 @@ void Promote(OBJECT x, OBJECT stop_link, OBJECT dest_index)
 	    /* galley is part flushed, leave it here */
 	    link = NextDown(link);
 	  }
-	  else if( backward(z) )
+	  else if( foll_or_prec(z) == GALL_PREC )
 	  {
-	    /* galley is preceding, send to CrossSequence */
+	    /* galley is preceding or foll_or_prec, send to CrossSequence */
 	    OBJECT t;
 	    type(y) = GALL_PREC;
 	    pinpoint(y) = nilobj;
@@ -397,9 +491,11 @@ void Promote(OBJECT x, OBJECT stop_link, OBJECT dest_index)
 	  else
 	  {
 	    /* galley was never attached, print message and kill it */
-	    Error(22, 6, "galley %s deleted (never attached)",
+	    Error(22, 4, "galley %s deleted (never attached)",
 	      WARN, &fpos(z), SymName(actual(z)));
-	    KillGalley(z);
+	    debug1(DGF, D, "never-attached galley %s:", EchoFilePos(&fpos(z)));
+	    ifdebug(DGF, D, DebugObject(z));
+	    KillGalley(z, FALSE);
 	  }
 	  break;
 
@@ -407,14 +503,14 @@ void Promote(OBJECT x, OBJECT stop_link, OBJECT dest_index)
 	case EXPAND_IND:
       
 	  /* expand @HExpand or @VExpand to occupy everything possible */
-	  dim = type(actual(y)) == HEXPAND ? COL : ROW;
+	  dim = type(actual(y)) == HEXPAND ? COLM : ROWM;
           debug1(DGP, D, " flushing %s", EchoObject(y));
-	  Constrained(actual(y), &c, dim);
+	  Constrained(actual(y), &c, dim, &why);
 	  if( constrained(c) )
 	  { LENGTH b = back(actual(y), dim);
 	    LENGTH f = fwd(actual(y), dim);
 	    EnlargeToConstraint(&b, &f, &c);
-	    debug2(DGP, D, "FlushRoot call AdjustSize(x, %s,%s, dim)",
+	    debug2(DGP, D, "FlushRoot call AdjustSize(hd, %s,%s, dim)",
 			EchoLength(b), EchoLength(f));
 	    debug1(DSA, D, "Promote %s AdjustSize", Image(type(actual(y))));
 	    AdjustSize(actual(y), b, f, dim);
@@ -440,13 +536,19 @@ void Promote(OBJECT x, OBJECT stop_link, OBJECT dest_index)
 	  break;
 
 
-	case GALL_PREC:
-	case GALL_FOLL:
-	case GALL_TARG:
 	case CROSS_PREC:
 	case CROSS_FOLL:
 	case CROSS_TARG:
 	      
+	  debug2(DGS, D, "root promote %s %s", Image(type(y)), EchoObject(y));
+	  /* NB NO BREAK */
+
+
+	case GALL_PREC:
+	case GALL_FOLL:
+	case GALL_FOLL_OR_PREC:
+	case GALL_TARG:
+
 	  CrossSequence(actual(y));
 	  DisposeChild(NextDown(link));
 	  break;
@@ -462,13 +564,12 @@ void Promote(OBJECT x, OBJECT stop_link, OBJECT dest_index)
 	case VSHIFT:
 	case HSCALE:
 	case VSCALE:
+	case HCOVER:
+	case VCOVER:
 	case HCONTRACT:
 	case VCONTRACT:
 	case HEXPAND:
 	case VEXPAND:
-	case PADJUST:
-	case HADJUST:
-	case VADJUST:
 	case ROTATE:
 	case SCALE:
 	case INCGRAPHIC:
@@ -484,25 +585,28 @@ void Promote(OBJECT x, OBJECT stop_link, OBJECT dest_index)
 	case CROSS:
 
 	  /* print this component */
-	  debug0(DCR, D, "Promote --");
-	  if( !is_indefinite(type(y)) && size(y, ROW) != 0 )
+	  debug0(DGS, D, "root promote definite or indefinite");
+	  if( !is_indefinite(type(y)) && size(y, ROWM) != 0 )
 	  {
 	    /* fix horizontally; work out which fonts needed */
 	    /* *** nice in theory (preserves joins), but ... 
-	    FixAndPrintObject(y, back(x, COL), back(x, COL), fwd(x, COL),
-	      COL, LAST_ADJUST, FALSE, LAST_ADJUST, 0, 0);
+	    FixAndPrintObject(y, back(hd, COLM), back(hd, COLM), fwd(hd, COLM),
+	      COLM, FALSE, 0, 0);
 	    *** */
-	    FixAndPrintObject(y, back(y, COL), back(y, COL), fwd(y, COL),
-	      COL, LAST_ADJUST, FALSE, LAST_ADJUST, 0, 0);
+	    FixAndPrintObject(y, back(y, COLM), back(y, COLM), fwd(y, COLM),
+	      COLM, FALSE, 0, 0);
 
 	    /* print prefatory or page separating material, including fonts */
 	    label_string = page_label != nilobj && is_word(type(page_label)) ?
 	      string(page_label) : AsciiToFull("?");
+	    debug1(DGS, D, "root promote definite; label_string = %s",
+	      label_string);
+	    debug1(DCR, D, "label_string = %s", label_string);
 	    if( first )
-	    { PrintBeforeFirst(size(x, COL), size(y, ROW), label_string);
+	    { PrintBeforeFirst(size(hd, COLM), size(y, ROWM), label_string);
 	      first = FALSE;
 	    }
-	    else PrintBetween(size(x, COL), size(y, ROW), label_string);
+	    else PrintBetween(size(hd, COLM), size(y, ROWM), label_string);
 	    if( page_label != nilobj )
 	    { DisposeObject(page_label);
 	      page_label = nilobj;
@@ -510,8 +614,8 @@ void Promote(OBJECT x, OBJECT stop_link, OBJECT dest_index)
 
 	    /* fix and print vertically */
 	    debug1(DGF,D, "  Promote calling FixAndPrint %s", Image(type(y)));
-	    FixAndPrintObject(y, back(y,ROW), back(y, ROW), fwd(y, ROW),
-	      ROW, LAST_ADJUST, FALSE, LAST_ADJUST, size(y,ROW), 0);
+	    FixAndPrintObject(y, back(y,ROWM), back(y, ROWM), fwd(y, ROWM),
+	      ROWM, FALSE, size(y, ROWM), 0);
 
 	  }
 	  DisposeChild(NextDown(link));
@@ -529,51 +633,55 @@ void Promote(OBJECT x, OBJECT stop_link, OBJECT dest_index)
 
 	default:
       
-	  Error(22, 7, "Promote: %s", INTERN, &fpos(y), Image(type(y)));
+	  assert1(FALSE, "Promote:", Image(type(y)));
 	  break;
 	
       }
     }
-    debug0(DGS, D, "Promote returning (root galley).");
+    debug0(DGS, DD, "Promote returning (root galley).");
     return;
   }
 
   /* prepare the promotion */
-  if( external(dest) )
+  if( external_ver(dest) && gall_dir(hd) == ROWM )
   { if( threaded(dest) )
-    { Parent(tmp1, UpDim(dest, COL));
+    { Parent(tmp1, UpDim(dest, COLM));
       assert( type(tmp1) == COL_THR, "Promote: tmp1 not COL_THR!" );
-      y = FindSplitInGalley(x);
+      y = FindSplitInGalley(hd);
       assert( type(y) == SPLIT, "Promote: FindSplitInGalley!" );
-      Child(tmp2, DownDim(y, COL));
+      Child(tmp2, DownDim(y, COLM));
       assert( type(tmp2) == COL_THR, "Promote: tmp2 not COL_THR!" );
       if( tmp1 != tmp2 )
-      { LENGTH b = max(back(tmp1, COL), back(tmp2, COL));
-	LENGTH f = max(fwd(tmp1, COL),  fwd(tmp2, COL));
+      { LENGTH b = max(back(tmp1, COLM), back(tmp2, COLM));
+	LENGTH f = max(fwd(tmp1, COLM),  fwd(tmp2, COLM));
 	debug0(DSA, D, "calling AdjustSize(tmp1) from Promote (node merging)");
-	AdjustSize(tmp1, b, f, COL);
+	AdjustSize(tmp1, b, f, COLM);
 	debug0(DSA, D, "calling AdjustSize(tmp2) from Promote (node merging)");
-	AdjustSize(tmp2, b, f, COL);
+	AdjustSize(tmp2, b, f, COLM);
 	MergeNode(tmp1, tmp2);
       }
     }
     link = Up(dest_index);
   }
+  else if( external_hor(dest) && gall_dir(hd) == COLM )
+  { link = Up(dest_index);
+  }
   else
-  { for( link = x;  NextDown(link) != stop_link;  )
+  { for( link = hd;  NextDown(link) != stop_link;  )
     { Child(y, NextDown(link));
-      if( is_index(type(y)) )  MoveLink(NextDown(link), Up(dest_index), PARENT);
+      if( is_index(type(y)) )
+	MoveLink(NextDown(link), Up(dest_index), PARENT);
       else link = NextDown(link);
     }
-    assert( Down(x) != stop_link, "Promote: Down(x) == stop_link!" );
-    assert( UpDim(dest, ROW) == UpDim(dest, COL), "Promote: dims!" );
+    assert( Down(hd) != stop_link, "Promote: Down(hd) == stop_link!" );
+    assert( UpDim(dest, ROWM) == UpDim(dest, COLM), "Promote: dims!" );
     link = Up(dest);
   }
   
   /* promote components */
-  TransferLinks(Down(x), stop_link, link);
+  TransferLinks(Down(hd), stop_link, link);
 
-  debug0(DGS, D, "Promote returning.");
+  debug0(DGS, DD, "Promote returning.");
 } /* end Promote */
 
 
@@ -590,10 +698,12 @@ static void MakeDead(OBJECT y)
   static OBJECT	dead_store = nilobj;	/* where DEAD objects are kept       */
 
   debug1(DGS, DDD, "MakeDead( %s )", Image(type(y)));
-  if( dead_store == nilobj )  dead_store = New(ACAT);
+  if( dead_store == nilobj )  New(dead_store, ACAT);
   type(y) = DEAD;
   MoveLink(Up(y), dead_store, PARENT);
-  if( dead_count >= 150 )  DisposeChild(Down(dead_store));
+  if( dead_count >= 150 )
+  { DisposeChild(Down(dead_store));
+  }
   else dead_count++;
   debug1(DGS, DDD, "MakeDead returning (dead_count = %d).", dead_count);
 } /* end MakeDead */
@@ -601,14 +711,18 @@ static void MakeDead(OBJECT y)
 
 /*****************************************************************************/
 /*                                                                           */
-/*  KillGalley(hd)                                                           */
+/*  KillGalley(hd, optimize)                                                 */
 /*                                                                           */
 /*  Kill galley hd, which may be sized or unsized.  The index of hd must     */
 /*  be UNATTACHED; it is moved out of its present location to a secret spot. */
 /*                                                                           */
+/*  If hd is to be optimized, generate all the stuff for the cross           */
+/*  reference database.  However, don't do this if optimize is FALSE, for    */
+/*  in that case hd is defective in some way and not optimizable.            */
+/*                                                                           */
 /*****************************************************************************/
 
-void KillGalley(OBJECT hd)
+void KillGalley(OBJECT hd, BOOLEAN optimize)
 { OBJECT prnt, link, y, z;
   debug2(DGF, D, "[ KillGalley(Galley %s into %s)",
 	SymName(actual(hd)), SymName(whereto(hd)));
@@ -617,10 +731,13 @@ void KillGalley(OBJECT hd)
   assert( type(prnt) == UNATTACHED, "KillGalley: UNATTACHED precondition!" );
   assert( Up(prnt) != prnt, "KillGalley: prnt!" );
 
+  /* delete any ready_galls that might be hanging about */
   if( ready_galls(hd) != nilobj )
   { DisposeObject(ready_galls(hd));
     ready_galls(hd) = nilobj;
   }
+
+  /* delete every remaining component */
   for( link = hd; NextDown(link) != hd; )
   { Child(y, NextDown(link));
     switch( type(y) )
@@ -637,10 +754,10 @@ void KillGalley(OBJECT hd)
 			break;
 
       case UNATTACHED:	assert( Down(y) != y, "KillGalley: UNATTACHED!" );
-			Child(z, Down(y));  KillGalley(z);
+			Child(z, Down(y));  KillGalley(z, FALSE);
 			break;
 
-      case HEAD:	Error(22, 8, "KillGalley: head", INTERN, &fpos(y));
+      case HEAD:	assert(FALSE, "KillGalley: head");
 			break;
 
       default:		DisposeChild(NextDown(link));
@@ -648,6 +765,10 @@ void KillGalley(OBJECT hd)
     }
   }
 
+  /* perform optimization calculations if required */
+  if( optimize && opt_components(hd) != nilobj )
+    CalculateOptimize(hd);
+
   /* move index into dead_store */
   MakeDead(prnt);
   debug0(DGF, D, "] KillGalley returning.");
@@ -661,7 +782,7 @@ void KillGalley(OBJECT hd)
 /*  Free galley hd up to but not including stop_link.  *Inners is well-      */
 /*  defined, either nilobj or an ACAT of galleys to be flushed.              */
 /*                                                                           */
-/*  Relocate_link defines what to do any galley attached to one of the       */
+/*  Relocate_link defines what to do with any galley attached to one of the  */
 /*  freed targets.  If it is non-nilobj, galley hd is searched onwards from  */
 /*  it to see if a target can be found there.  If so, the galley is          */
 /*  relocated to just before that point.  If not, or if relocate_link is     */
@@ -686,7 +807,7 @@ OBJECT relocate_link, OBJECT sym)
   for( link = Down(hd);  link != stop_link;  link = NextDown(link) )
   { Child(y, link);
     if( type(y) == RECEIVING && actual(actual(y)) == InputSym )
-      Error(22, 9, "forcing galley after input point", WARN, &fpos(actual(y)));
+      Error(22, 5, "forcing galley after input point", WARN, &fpos(actual(y)));
     else if( type(y) == RECEIVING )
     {
       /* either relocate or free each galley */
@@ -698,20 +819,24 @@ OBJECT relocate_link, OBJECT sym)
 	if( relocate_link != nilobj && whereto(z) != sym &&
 	    (srch = SearchGalley(relocate_link, whereto(z), TRUE,
 	    FALSE, TRUE, FALSE)) != nilobj )
-	{ DetachGalley(z);
+	{
+	  if( opt_components(z) != nilobj )  GazumpOptimize(z, actual(y));
+	  DetachGalley(z);
 	  Parent(index, Up(z));
 	  MoveLink(Up(index), Up(srch), PARENT);  /* just before new dest */
 	}
 	else
 	{ debug0(DGA, D, "  calling FreeGalley from FreeGalley");
 	  FreeGalley(z, z, inners, nilobj, sym);
-	  if( *inners == nilobj )  *inners = New(ACAT);
+	  if( *inners == nilobj )  New(*inners, ACAT);
 	  Link(*inners, y);
 	}
       }
       non_blocking(y) = TRUE;
     }
-    else if( type(y) == RECEPTIVE )  non_blocking(y) = TRUE;
+    else if( type(y) == RECEPTIVE )
+    { non_blocking(y) = TRUE;
+    }
   }
   debug0(DGA, D, "] FreeGalley returning.");
 } /* end FreeGalley */
@@ -719,16 +844,16 @@ OBJECT relocate_link, OBJECT sym)
 
 /*@::TargetSymbol()@**********************************************************/
 /*                                                                           */
-/*  BOOLEAN TargetSymbol(x, sym)                                             */
+/*  unsigned char TargetSymbol(x, sym)                                       */
 /*                                                                           */
 /*  Examine the parameters of closure x, which is known to have a @Target.   */
-/*  Return TRUE if the target is preceding, and set sym to the symbol value. */
+/*  Return GALL_PREC, GALL_FOLL, or GALL_PREC_OR_FOLL, depending.            */
 /*                                                                           */
 /*****************************************************************************/
 
-BOOLEAN TargetSymbol(OBJECT x, OBJECT *sym)
-{ OBJECT y, link, cr, lpar, rpar;
-  debug1(DGS, D, "TargetSymbol( %s )", EchoObject(x));
+unsigned char TargetSymbol(OBJECT x, OBJECT *sym)
+{ OBJECT y, link, cr, lpar, rpar;  unsigned char res;
+  debug1(DGS, DD, "TargetSymbol( %s )", EchoObject(x));
   assert( type(x) == CLOSURE, "TargetSymbol: type(x) != CLOSURE!" );
   assert( has_target(actual(x)), "TargetSymbol: x has no target!" );
 
@@ -759,29 +884,43 @@ BOOLEAN TargetSymbol(OBJECT x, OBJECT *sym)
     debug1(DGS, DD, "TargetSymbol examining %s", EchoObject(cr));
     debug1(DGS, DD, "  type(cr) = %s", Image( (int) type(cr)) );
     if( type(cr) != CROSS )
-      Error(22, 10, "target of %s is not a cross reference",
+      Error(22, 6, "target of %s is not a cross reference",
 	FATAL, &fpos(cr), SymName(actual(x)));
 
     /* extract *sym from the left parameter */
     Child(lpar, Down(cr));
     if( type(lpar) != CLOSURE )
-      Error(22, 11, "left parameter of %s is not a symbol",
+      Error(22, 7, "left parameter of %s is not a symbol",
 	FATAL, &fpos(lpar), KW_CROSS);
     *sym = actual(lpar);
 
     /* extract direction from the right parameter */
     Child(rpar, NextDown(Down(cr)));
-    if( !is_word(type(rpar)) ||
-	( !StringEqual(string(rpar), KW_PRECEDING) &&
-	  !StringEqual(string(rpar), KW_FOLLOWING) ) )
-      Error(22, 12, "replacing %s%s? by %s%s%s",
-	WARN, &fpos(rpar), SymName(actual(lpar)), KW_CROSS,
+    if( !is_word(type(rpar)) )
+    {
+      Error(22, 8, "replacing %s%s? by %s%s%s", WARN, &fpos(rpar),
+	SymName(actual(lpar)), KW_CROSS,
 	SymName(actual(lpar)), KW_CROSS, KW_FOLLOWING);
-    return is_word(type(rpar)) && StringEqual(string(rpar), KW_PRECEDING);
+      res = GALL_FOLL;
+    }
+    else if( StringEqual(string(rpar), KW_PRECEDING) )
+       res = GALL_PREC;
+    else if( StringEqual(string(rpar), KW_FOLLOWING) )
+       res = GALL_FOLL;
+    else if( StringEqual(string(rpar), KW_FOLL_OR_PREC) )
+       res = GALL_FOLL_OR_PREC;
+    else
+    {
+      Error(22, 9, "replacing %s%s%s by %s%s%s",
+	WARN, &fpos(rpar), SymName(actual(lpar)), KW_CROSS,
+	string(rpar), SymName(actual(lpar)), KW_CROSS, KW_FOLLOWING);
+      res = GALL_FOLL;
+    }
+    return res;
   }
   else
-  { Error(22, 13, "TargetSymbol: missing target of x", INTERN, &fpos(x));
-    return FALSE;
+  { assert(FALSE, "TargetSymbol: missing target of x");
+    return GALL_FOLL;
   }
 } /* end TargetSymbol */
 
@@ -802,7 +941,7 @@ BOOLEAN TargetSymbol(OBJECT x, OBJECT *sym)
 
 int CheckComponentOrder(OBJECT preceder, OBJECT follower)
 { OBJECT prec_galley, foll_galley, z;  int res;
-  debug2(DGS, D, "CheckComponentOrder( %s, %s )",
+  debug2(DGS, DD, "CheckComponentOrder( %s, %s )",
     EchoObject(preceder), EchoObject(follower));
   Parent(prec_galley, Up(preceder));
   Parent(foll_galley, Up(follower));
@@ -825,6 +964,6 @@ int CheckComponentOrder(OBJECT preceder, OBJECT follower)
       }
     }
   }
-  debug1(DGS, D, "CheckComponentOrder returning %s", Image(res));
+  debug1(DGS, DD, "CheckComponentOrder returning %s", Image(res));
   return res;
 } /* end CheckComponentOrder */
