@@ -1,9 +1,9 @@
 /*@z30.c:Symbol uses:InsertUses()@********************************************/
 /*                                                                           */
-/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.06)                       */
-/*  COPYRIGHT (C) 1994 Jeffrey H. Kingston                                   */
+/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.08)                       */
+/*  COPYRIGHT (C) 1991, 1996 Jeffrey H. Kingston                             */
 /*                                                                           */
-/*  Jeffrey H. Kingston (jeff@cs.su.oz.au)                                   */
+/*  Jeffrey H. Kingston (jeff@cs.usyd.edu.au)                                */
 /*  Basser Department of Computer Science                                    */
 /*  The University of Sydney 2006                                            */
 /*  AUSTRALIA                                                                */
@@ -44,7 +44,7 @@ void InsertUses(OBJECT x, OBJECT y)
 { OBJECT tmp;
   debug2(DSU, D, "InsertUses( %s, %s )", SymName(x), SymName(y));
   if( type(x) == LOCAL && type(y) == LOCAL && !predefined(y) )
-  { tmp = GetMem(USES_SIZE, no_fpos);  item(tmp) = y;
+  { GetMem(tmp, USES_SIZE, no_fpos);  item(tmp) = y;
     if( base_uses(x) == nilobj )  next(tmp) = tmp;
     else next(tmp) = next(base_uses(x)), next(base_uses(x)) = tmp;
     base_uses(x) = tmp;
@@ -79,7 +79,7 @@ static void GatherUses(OBJECT x, OBJECT sym)
       if( marker(y) != sym )
       {	if( y != sym )
 	{ marker(y) = sym;
-	  tmp = GetMem(USES_SIZE, no_fpos);  item(tmp) = y;
+	  GetMem(tmp, USES_SIZE, no_fpos);  item(tmp) = y;
 	  if( uses(sym) == nilobj )  next(tmp) = tmp;
 	  else next(tmp) = next(uses(sym)), next(uses(sym)) = tmp;
 	  uses(sym) = tmp;
